@@ -6,7 +6,7 @@
  * MW_NO_SETUP is defined.
  *
  * Setup.php (if loaded) then sets up GlobalFunctions, the AutoLoader,
- * and the configuration globals (though not $wgTitle).
+ * and the configuration globals.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,38 +26,22 @@
  * @file
  */
 
-# Protect against register_globals
+# Die if register_globals is enabled (PHP <=5.3)
 # This must be done before any globals are set by the code
 if ( ini_get( 'register_globals' ) ) {
-	if ( isset( $_REQUEST['GLOBALS'] ) || isset( $_FILES['GLOBALS'] ) ) {
-		die( '<a href="http://www.hardened-php.net/globals-problem">$GLOBALS overwrite vulnerability</a>' );
-	}
-	$verboten = array(
-		'GLOBALS',
-		'_SERVER',
-		'HTTP_SERVER_VARS',
-		'_GET',
-		'HTTP_GET_VARS',
-		'_POST',
-		'HTTP_POST_VARS',
-		'_COOKIE',
-		'HTTP_COOKIE_VARS',
-		'_FILES',
-		'HTTP_POST_FILES',
-		'_ENV',
-		'HTTP_ENV_VARS',
-		'_REQUEST',
-		'_SESSION',
-		'HTTP_SESSION_VARS'
-	);
-	foreach ( $_REQUEST as $name => $value ) {
-		if ( in_array( $name, $verboten ) ) {
-			header( "HTTP/1.1 500 Internal Server Error" );
-			echo "register_globals security paranoia: trying to overwrite superglobals, aborting.";
-			die( -1 );
-		}
-		unset( $GLOBALS[$name] );
-	}
+	die( 'MediaWiki does not support installations where register_globals is enabled. '
+		. 'Please see <a href="https://www.mediawiki.org/wiki/register_globals">mediawiki.org</a> '
+		. 'for help on how to disable it.' );
+}
+
+if ( function_exists( 'get_magic_quotes_gpc' ) && get_magic_quotes_gpc() ) {
+	die( 'MediaWiki does not function when magic quotes are enabled. '
+		. 'Please see the <a href="https://php.net/manual/security.magicquotes.disabling.php">PHP Manual</a> '
+		. 'for help on how to disable magic quotes.' );
+}
+
+if ( ini_get( 'mbstring.func_overload' ) ) {
+       die( 'MediaWiki does not support installations where mbstring.func_overload is non-zero.' );
 }
 
 # bug 15461: Make IE8 turn off content sniffing. Everybody else should ignore this
@@ -65,13 +49,18 @@ if ( ini_get( 'register_globals' ) ) {
 # points and when $wgOut gets disabled or overridden.
 header( 'X-Content-Type-Options: nosniff' );
 
-$wgRequestTime = microtime( true );
-# getrusage() does not exist on the Microsoft Windows platforms, catching this
-if ( function_exists ( 'getrusage' ) ) {
-	$wgRUstart = getrusage();
-} else {
-	$wgRUstart = array();
+# Approximate $_SERVER['REQUEST_TIME_FLOAT'] for PHP<5.4
+if ( !isset( $_SERVER['REQUEST_TIME_FLOAT'] ) ) {
+	$_SERVER['REQUEST_TIME_FLOAT'] = microtime( true );
 }
+
+/**
+ * @var float Request start time as fractional seconds since epoch
+ * @deprecated since 1.25; use $_SERVER['REQUEST_TIME_FLOAT'] or
+ *   WebRequest::getElapsedTime() instead.
+ */
+$wgRequestTime = $_SERVER['REQUEST_TIME_FLOAT'];
+
 unset( $IP );
 
 # Valid web server entry point, enable includes.
@@ -87,19 +76,16 @@ define( 'MEDIAWIKI', true );
 # if we don't have permissions on parent directories.
 $IP = getenv( 'MW_INSTALL_PATH' );
 if ( $IP === false ) {
-	if ( realpath( '.' ) ) {
-		$IP = realpath( '.' );
-	} else {
-		$IP = dirname( __DIR__ );
-	}
+	$IP = realpath( '.' ) ?: dirname( __DIR__ );
 }
 
+# Grab profiling functions
+require_once "$IP/includes/profiler/ProfilerFunctions.php";
+$wgRUstart = wfGetRusage() ?: array();
+
 # Start the autoloader, so that extensions can derive classes from core files
 require_once "$IP/includes/AutoLoader.php";
 
-# Load the profiler
-require_once "$IP/includes/profiler/Profiler.php";
-
 # Load up some global defines.
 require_once "$IP/includes/Defines.php";
 
@@ -109,11 +95,13 @@ if ( file_exists( "$IP/StartProfiler.php" ) ) {
 	require "$IP/StartProfiler.php";
 }
 
-wfProfileIn( 'WebStart.php-conf' );
 
 # Load default settings
 require_once "$IP/includes/DefaultSettings.php";
 
+# Load global functions
+require_once "$IP/includes/GlobalFunctions.php";
+
 # Load composer's autoloader if present
 if ( is_readable( "$IP/vendor/autoload.php" ) ) {
 	require_once "$IP/vendor/autoload.php";
@@ -131,7 +119,7 @@ if ( defined( 'MW_CONFIG_CALLBACK' ) ) {
 	# the wiki installer needs to be launched or the generated file uploaded to
 	# the root wiki directory. Give a hint, if it is not readable by the server.
 	if ( !is_readable( MW_CONFIG_FILE ) ) {
-		require_once "$IP/includes/templates/NoLocalSettings.php";
+		require_once "$IP/includes/NoLocalSettings.php";
 		die();
 	}
 
@@ -139,18 +127,15 @@ if ( defined( 'MW_CONFIG_CALLBACK' ) ) {
 	require_once MW_CONFIG_FILE;
 }
 
-wfProfileOut( 'WebStart.php-conf' );
 
-wfProfileIn( 'WebStart.php-ob_start' );
 # Initialise output buffering
 # Check that there is no previous output or previously set up buffers, because
 # that would cause us to potentially mix gzip and non-gzip output, creating a
 # big mess.
-if ( !defined( 'MW_NO_OUTPUT_BUFFER' ) && ob_get_level() == 0 ) {
+if ( ob_get_level() == 0 ) {
 	require_once "$IP/includes/OutputHandler.php";
 	ob_start( 'wfOutputHandler' );
 }
-wfProfileOut( 'WebStart.php-ob_start' );
 
 if ( !defined( 'MW_NO_SETUP' ) ) {
 	require_once "$IP/includes/Setup.php";
