@@ -26,10 +26,10 @@ class ApiImageRotate extends ApiBase {
 
 	/**
 	 * Add all items from $values into the result
-	 * @param array $result output
-	 * @param array $values values to add
-	 * @param string $flag the name of the boolean flag to mark this element
-	 * @param string $name if given, name of the value
+	 * @param array $result Output
+	 * @param array $values Values to add
+	 * @param string $flag The name of the boolean flag to mark this element
+	 * @param string $name If given, name of the value
 	 */
 	private static function addValues( array &$result, $values, $flag = null, $name = null ) {
 		foreach ( $values as $val ) {
@@ -42,7 +42,7 @@ class ApiImageRotate extends ApiBase {
 				$v = $val;
 			}
 			if ( $flag !== null ) {
-				$v[$flag] = '';
+				$v[$flag] = true;
 			}
 			$result[] = $v;
 		}
@@ -52,6 +52,9 @@ class ApiImageRotate extends ApiBase {
 		$params = $this->extractRequestParams();
 		$rotation = $params['rotation'];
 
+		$continuationManager = new ApiContinuationManager( $this, array(), array() );
+		$this->setContinuationManager( $continuationManager );
+
 		$pageSet = $this->getPageSet();
 		$pageSet->execute();
 
@@ -68,10 +71,10 @@ class ApiImageRotate extends ApiBase {
 			$r['id'] = $title->getArticleID();
 			ApiQueryBase::addTitleInfo( $r, $title );
 			if ( !$title->exists() ) {
-				$r['missing'] = '';
+				$r['missing'] = true;
 			}
 
-			$file = wfFindFile( $title );
+			$file = wfFindFile( $title, array( 'latest' => true ) );
 			if ( !$file ) {
 				$r['result'] = 'Failure';
 				$r['errormessage'] = 'File does not exist';
@@ -120,7 +123,7 @@ class ApiImageRotate extends ApiBase {
 					$r['result'] = 'Success';
 				} else {
 					$r['result'] = 'Failure';
-					$r['errormessage'] = $this->getResult()->convertStatusToArray( $status );
+					$r['errormessage'] = $this->getErrorFormatter()->arrayFromStatus( $status );
 				}
 			} else {
 				$r['result'] = 'Failure';
@@ -129,8 +132,11 @@ class ApiImageRotate extends ApiBase {
 			$result[] = $r;
 		}
 		$apiResult = $this->getResult();
-		$apiResult->setIndexedTagName( $result, 'page' );
+		ApiResult::setIndexedTagName( $result, 'page' );
 		$apiResult->addValue( null, $this->getModuleName(), $result );
+
+		$this->setContinuationManager( null );
+		$continuationManager->setContinuationIntoResult( $apiResult );
 	}
 
 	/**
@@ -181,9 +187,8 @@ class ApiImageRotate extends ApiBase {
 				ApiBase::PARAM_TYPE => array( '90', '180', '270' ),
 				ApiBase::PARAM_REQUIRED => true
 			),
-			'token' => array(
-				ApiBase::PARAM_TYPE => 'string',
-				ApiBase::PARAM_REQUIRED => true
+			'continue' => array(
+				ApiBase::PARAM_HELP_MSG => 'api-help-param-continue',
 			),
 		);
 		if ( $flags ) {
@@ -193,39 +198,17 @@ class ApiImageRotate extends ApiBase {
 		return $result;
 	}
 
-	public function getParamDescription() {
-		$pageSet = $this->getPageSet();
-
-		return $pageSet->getFinalParamDescription() + array(
-			'rotation' => 'Degrees to rotate image clockwise',
-			'token' => 'Edit token. You can get one of these through action=tokens',
-		);
-	}
-
-	public function getDescription() {
-		return 'Rotate one or more images.';
-	}
-
 	public function needsToken() {
-		return true;
+		return 'csrf';
 	}
 
-	public function getTokenSalt() {
-		return '';
-	}
-
-	public function getPossibleErrors() {
-		$pageSet = $this->getPageSet();
-
-		return array_merge(
-			parent::getPossibleErrors(),
-			$pageSet->getFinalPossibleErrors()
-		);
-	}
-
-	public function getExamples() {
+	protected function getExamplesMessages() {
 		return array(
-			'api.php?action=imagerotate&titles=Example.jpg&rotation=90&token=123ABC',
+			'action=imagerotate&titles=File:Example.jpg&rotation=90&token=123ABC'
+				=> 'apihelp-imagerotate-example-simple',
+			'action=imagerotate&generator=categorymembers&gcmtitle=Category:Flip&gcmtype=file&' .
+				'rotation=180&token=123ABC'
+				=> 'apihelp-imagerotate-example-generator',
 		);
 	}
 }
