@@ -70,11 +70,16 @@ class ApiMove extends ApiBase {
 			}
 		}
 
+		// Rate limit
+		if ( $user->pingLimiter( 'move' ) ) {
+			$this->dieUsageMsg( 'actionthrottledtext' );
+		}
+
 		// Move the page
 		$toTitleExists = $toTitle->exists();
-		$retval = $fromTitle->moveTo( $toTitle, true, $params['reason'], !$params['noredirect'] );
-		if ( $retval !== true ) {
-			$this->dieUsageMsg( reset( $retval ) );
+		$status = $this->movePage( $fromTitle, $toTitle, $params['reason'], !$params['noredirect'] );
+		if ( !$status->isOK() ) {
+			$this->dieStatus( $status );
 		}
 
 		$r = array(
@@ -83,34 +88,28 @@ class ApiMove extends ApiBase {
 			'reason' => $params['reason']
 		);
 
-		if ( $fromTitle->exists() ) {
-			//NOTE: we assume that if the old title exists, it's because it was re-created as
-			// a redirect to the new title. This is not safe, but what we did before was
-			// even worse: we just determined whether a redirect should have been created,
-			// and reported that it was created if it should have, without any checks.
-			// Also note that isRedirect() is unreliable because of bug 37209.
-			$r['redirectcreated'] = '';
-		}
+		//NOTE: we assume that if the old title exists, it's because it was re-created as
+		// a redirect to the new title. This is not safe, but what we did before was
+		// even worse: we just determined whether a redirect should have been created,
+		// and reported that it was created if it should have, without any checks.
+		// Also note that isRedirect() is unreliable because of bug 37209.
+		$r['redirectcreated'] = $fromTitle->exists();
 
-		if ( $toTitleExists ) {
-			$r['moveoverredirect'] = '';
-		}
+		$r['moveoverredirect'] = $toTitleExists;
 
 		// Move the talk page
 		if ( $params['movetalk'] && $fromTalk->exists() && !$fromTitle->isTalkPage() ) {
 			$toTalkExists = $toTalk->exists();
-			$retval = $fromTalk->moveTo( $toTalk, true, $params['reason'], !$params['noredirect'] );
-			if ( $retval === true ) {
+			$status = $this->movePage( $fromTalk, $toTalk, $params['reason'], !$params['noredirect'] );
+			if ( $status->isOK() ) {
 				$r['talkfrom'] = $fromTalk->getPrefixedText();
 				$r['talkto'] = $toTalk->getPrefixedText();
-				if ( $toTalkExists ) {
-					$r['talkmoveoverredirect'] = '';
-				}
+				$r['talkmoveoverredirect'] = $toTalkExists;
 			} else {
 				// We're not gonna dieUsage() on failure, since we already changed something
-				$parsed = $this->parseMsg( reset( $retval ) );
-				$r['talkmove-error-code'] = $parsed['code'];
-				$r['talkmove-error-info'] = $parsed['info'];
+				$error = $this->getErrorFromStatus( $status );
+				$r['talkmove-error-code'] = $error[0];
+				$r['talkmove-error-info'] = $error[1];
 			}
 		}
 
@@ -120,12 +119,12 @@ class ApiMove extends ApiBase {
 		if ( $params['movesubpages'] ) {
 			$r['subpages'] = $this->moveSubpages( $fromTitle, $toTitle,
 				$params['reason'], $params['noredirect'] );
-			$result->setIndexedTagName( $r['subpages'], 'subpage' );
+			ApiResult::setIndexedTagName( $r['subpages'], 'subpage' );
 
 			if ( $params['movetalk'] ) {
 				$r['subpages-talk'] = $this->moveSubpages( $fromTalk, $toTalk,
 					$params['reason'], $params['noredirect'] );
-				$result->setIndexedTagName( $r['subpages-talk'], 'subpage' );
+				ApiResult::setIndexedTagName( $r['subpages-talk'], 'subpage' );
 			}
 		}
 
@@ -134,8 +133,10 @@ class ApiMove extends ApiBase {
 			$watch = $params['watchlist'];
 		} elseif ( $params['watch'] ) {
 			$watch = 'watch';
+			$this->logFeatureUsage( 'action=move&watch' );
 		} elseif ( $params['unwatch'] ) {
 			$watch = 'unwatch';
+			$this->logFeatureUsage( 'action=move&unwatch' );
 		}
 
 		// Watch pages
@@ -145,11 +146,38 @@ class ApiMove extends ApiBase {
 		$result->addValue( null, $this->getModuleName(), $r );
 	}
 
+	/**
+	 * @param Title $from
+	 * @param Title $to
+	 * @param string $reason
+	 * @param bool $createRedirect
+	 * @return Status
+	 */
+	protected function movePage( Title $from, Title $to, $reason, $createRedirect ) {
+		$mp = new MovePage( $from, $to );
+		$valid = $mp->isValidMove();
+		if ( !$valid->isOK() ) {
+			return $valid;
+		}
+
+		$permStatus = $mp->checkPermissions( $this->getUser(), $reason );
+		if ( !$permStatus->isOK() ) {
+			return $permStatus;
+		}
+
+		// Check suppressredirect permission
+		if ( !$this->getUser()->isAllowed( 'suppressredirect' ) ) {
+			$createRedirect = true;
+		}
+
+		return $mp->move( $this->getUser(), $reason, $createRedirect );
+	}
+
 	/**
 	 * @param Title $fromTitle
 	 * @param Title $toTitle
-	 * @param $reason
-	 * @param $noredirect
+	 * @param string $reason
+	 * @param bool $noredirect
 	 * @return array
 	 */
 	public function moveSubpages( $fromTitle, $toTitle, $reason, $noredirect ) {
@@ -193,10 +221,6 @@ class ApiMove extends ApiBase {
 				ApiBase::PARAM_TYPE => 'string',
 				ApiBase::PARAM_REQUIRED => true
 			),
-			'token' => array(
-				ApiBase::PARAM_TYPE => 'string',
-				ApiBase::PARAM_REQUIRED => true
-			),
 			'reason' => '',
 			'movetalk' => false,
 			'movesubpages' => false,
@@ -222,84 +246,15 @@ class ApiMove extends ApiBase {
 		);
 	}
 
-	public function getParamDescription() {
-		$p = $this->getModulePrefix();
-
-		return array(
-			'from' => "Title of the page you want to move. Cannot be used together with {$p}fromid",
-			'fromid' => "Page ID of the page you want to move. Cannot be used together with {$p}from",
-			'to' => 'Title you want to rename the page to',
-			'token' => 'A move token previously retrieved through prop=info',
-			'reason' => 'Reason for the move',
-			'movetalk' => 'Move the talk page, if it exists',
-			'movesubpages' => 'Move subpages, if applicable',
-			'noredirect' => 'Don\'t create a redirect',
-			'watch' => 'Add the page and the redirect to your watchlist',
-			'unwatch' => 'Remove the page and the redirect from your watchlist',
-			'watchlist' => 'Unconditionally add or remove the page from your ' .
-				'watchlist, use preferences or do not change watch',
-			'ignorewarnings' => 'Ignore any warnings'
-		);
-	}
-
-	public function getResultProperties() {
-		return array(
-			'' => array(
-				'from' => 'string',
-				'to' => 'string',
-				'reason' => 'string',
-				'redirectcreated' => 'boolean',
-				'moveoverredirect' => 'boolean',
-				'talkfrom' => array(
-					ApiBase::PROP_TYPE => 'string',
-					ApiBase::PROP_NULLABLE => true
-				),
-				'talkto' => array(
-					ApiBase::PROP_TYPE => 'string',
-					ApiBase::PROP_NULLABLE => true
-				),
-				'talkmoveoverredirect' => 'boolean',
-				'talkmove-error-code' => array(
-					ApiBase::PROP_TYPE => 'string',
-					ApiBase::PROP_NULLABLE => true
-				),
-				'talkmove-error-info' => array(
-					ApiBase::PROP_TYPE => 'string',
-					ApiBase::PROP_NULLABLE => true
-				)
-			)
-		);
-	}
-
-	public function getDescription() {
-		return 'Move a page.';
-	}
-
-	public function getPossibleErrors() {
-		return array_merge( parent::getPossibleErrors(),
-			$this->getRequireOnlyOneParameterErrorMessages( array( 'from', 'fromid' ) ),
-			array(
-				array( 'invalidtitle', 'from' ),
-				array( 'nosuchpageid', 'fromid' ),
-				array( 'notanarticle' ),
-				array( 'invalidtitle', 'to' ),
-				array( 'sharedfile-exists' ),
-			)
-		);
-	}
-
 	public function needsToken() {
-		return true;
+		return 'csrf';
 	}
 
-	public function getTokenSalt() {
-		return '';
-	}
-
-	public function getExamples() {
+	protected function getExamplesMessages() {
 		return array(
-			'api.php?action=move&from=Badtitle&to=Goodtitle&token=123ABC&' .
+			'action=move&from=Badtitle&to=Goodtitle&token=123ABC&' .
 				'reason=Misspelled%20title&movetalk=&noredirect='
+				=> 'apihelp-move-example-move',
 		);
 	}
 
