@@ -111,7 +111,7 @@ class ApiRevisionDelete extends ApiBase {
 		// @codingStandardsIgnoreEnd
 
 		$data['items'] = array_values( $data['items'] );
-		$result->setIndexedTagName( $data['items'], 'i' );
+		ApiResult::setIndexedTagName( $data['items'], 'i' );
 		$result->addValue( null, $this->getModuleName(), $data );
 	}
 
@@ -121,12 +121,12 @@ class ApiRevisionDelete extends ApiBase {
 		);
 		$errors = $this->formatStatusMessages( $status->getErrorsByType( 'error' ) );
 		if ( $errors ) {
-			$this->getResult()->setIndexedTagName( $errors, 'e' );
+			ApiResult::setIndexedTagName( $errors, 'e' );
 			$ret['errors'] = $errors;
 		}
 		$warnings = $this->formatStatusMessages( $status->getErrorsByType( 'warning' ) );
 		if ( $warnings ) {
-			$this->getResult()->setIndexedTagName( $warnings, 'w' );
+			ApiResult::setIndexedTagName( $warnings, 'w' );
 			$ret['warnings'] = $warnings;
 		}
 
@@ -146,14 +146,14 @@ class ApiRevisionDelete extends ApiBase {
 				$message = array( 'message' => $msg->getKey() );
 				if ( $msg->getParams() ) {
 					$message['params'] = $msg->getParams();
-					$result->setIndexedTagName( $message['params'], 'p' );
+					ApiResult::setIndexedTagName( $message['params'], 'p' );
 				}
 			} else {
 				$message = array( 'message' => $m['message'] );
 				$msg = wfMessage( $m['message'] );
 				if ( isset( $m['params'] ) ) {
 					$message['params'] = $m['params'];
-					$result->setIndexedTagName( $message['params'], 'p' );
+					ApiResult::setIndexedTagName( $message['params'], 'p' );
 					$msg->params( $m['params'] );
 				}
 			}
@@ -195,57 +195,22 @@ class ApiRevisionDelete extends ApiBase {
 				ApiBase::PARAM_TYPE => array( 'yes', 'no', 'nochange' ),
 				ApiBase::PARAM_DFLT => 'nochange',
 			),
-			'token' => array(
-				ApiBase::PARAM_TYPE => 'string',
-				ApiBase::PARAM_REQUIRED => true
-			),
 			'reason' => null,
 		);
 	}
 
-	public function getParamDescription() {
-		return array(
-			'type' => 'Type of revision deletion being performed',
-			'target' => 'Page title for the revision deletion, if required for the type',
-			'ids' => 'Identifiers for the revisions to be deleted',
-			'hide' => 'What to hide for each revision',
-			'show' => 'What to unhide for each revision',
-			'suppress' => 'Whether to suppress data from administrators as well as others',
-			'token' => 'A delete token previously retrieved through action=tokens',
-			'reason' => 'Reason for the deletion/undeletion',
-		);
-	}
-
-	public function getDescription() {
-		return 'Delete/undelete revisions.';
-	}
-
-	public function getPossibleErrors() {
-		return array_merge( parent::getPossibleErrors(),
-			array(
-				array( 'code' => 'needtarget',
-					'info' => 'A target title is required for this RevDel type' ),
-				array( 'code' => 'badparams', 'info' => 'Bad value for some parameter' ),
-			)
-		);
-	}
-
 	public function needsToken() {
-		return true;
+		return 'csrf';
 	}
 
-	public function getTokenSalt() {
-		return '';
-	}
-
-	public function getExamples() {
+	protected function getExamplesMessages() {
 		return array(
-			'api.php?action=revisiondelete&target=Main%20Page&type=revision&ids=12345&' .
+			'action=revisiondelete&target=Main%20Page&type=revision&ids=12345&' .
 				'hide=content&token=123ABC'
-				=> 'Hide content for revision 12345 on the Main Page',
-			'api.php?action=revisiondelete&type=logging&ids=67890&hide=content|comment|user&' .
+				=> 'apihelp-revisiondelete-example-revision',
+			'action=revisiondelete&type=logging&ids=67890&hide=content|comment|user&' .
 				'reason=BLP%20violation&token=123ABC'
-				=> 'Hide all data on log entry 67890 with the reason "BLP violation"',
+				=> 'apihelp-revisiondelete-example-log',
 		);
 	}
 
