@@ -94,13 +94,17 @@ class DifferenceEngine extends ContextSource {
 
 	/** @var bool Show rev_deleted content if allowed */
 	protected $unhide = false;
+
+	/** @var bool Refresh the diff cache */
+	protected $mRefreshCache = false;
+
 	/**#@-*/
 
 	/**
 	 * Constructor
-	 * @param IContextSource $context context to use, anything else will be ignored
-	 * @param int $old old ID we want to show and diff with.
-	 * @param string|int $new either revision ID or 'prev' or 'next'. Default: 0.
+	 * @param IContextSource $context Context to use, anything else will be ignored
+	 * @param int $old Old ID we want to show and diff with.
+	 * @param string|int $new Either revision ID or 'prev' or 'next'. Default: 0.
 	 * @param int $rcid Deprecated, no longer used!
 	 * @param bool $refreshCache If set, refreshes the diff cache
 	 * @param bool $unhide If set, allow viewing deleted revs
@@ -224,12 +228,14 @@ class DifferenceEngine extends ContextSource {
 		}
 
 		$out->setPageTitle( $this->msg( 'errorpagetitle' ) );
-		$out->addWikiMsg( 'difference-missing-revision',
-			$this->getLanguage()->listToText( $missing ), count( $missing ) );
+		$msg = $this->msg( 'difference-missing-revision' )
+			->params( $this->getLanguage()->listToText( $missing ) )
+			->numParams( count( $missing ) )
+			->parseAsBlock();
+		$out->addHtml( $msg );
 	}
 
 	public function showDiffPage( $diffOnly = false ) {
-		wfProfileIn( __METHOD__ );
 
 		# Allow frames except in certain special cases
 		$out = $this->getOutput();
@@ -238,7 +244,6 @@ class DifferenceEngine extends ContextSource {
 
 		if ( !$this->loadRevisionData() ) {
 			$this->showMissingRevision();
-			wfProfileOut( __METHOD__ );
 
 			return;
 		}
@@ -250,7 +255,6 @@ class DifferenceEngine extends ContextSource {
 				$this->mOldPage->getUserPermissionsErrors( 'read', $user ) );
 		}
 		if ( count( $permErrors ) ) {
-			wfProfileOut( __METHOD__ );
 			throw new PermissionsError( 'read', $permErrors );
 		}
 
@@ -280,7 +284,7 @@ class DifferenceEngine extends ContextSource {
 			$samePage = true;
 			$oldHeader = '';
 		} else {
-			wfRunHooks( 'DiffViewHeader', array( $this, $this->mOldRev, $this->mNewRev ) );
+			Hooks::run( 'DiffViewHeader', array( $this, $this->mOldRev, $this->mNewRev ) );
 
 			if ( $this->mNewPage->equals( $this->mOldPage ) ) {
 				$out->setPageTitle( $this->msg( 'difference-title', $this->mNewPage->getPrefixedText() ) );
@@ -310,11 +314,11 @@ class DifferenceEngine extends ContextSource {
 								'undoafter' => $this->mOldid,
 								'undo' => $this->mNewid
 							) ),
-							'title' => Linker::titleAttrib( 'undo' )
+							'title' => Linker::titleAttrib( 'undo' ),
 						),
 						$this->msg( 'editundo' )->text()
 					);
-					$revisionTools[] = $undoLink;
+					$revisionTools['mw-diff-undo'] = $undoLink;
 				}
 			}
 
@@ -384,11 +388,17 @@ class DifferenceEngine extends ContextSource {
 		$rdel = $this->revisionDeleteLink( $this->mNewRev );
 
 		# Allow extensions to define their own revision tools
-		wfRunHooks( 'DiffRevisionTools', array( $this->mNewRev, &$revisionTools, $this->mOldRev ) );
+		Hooks::run( 'DiffRevisionTools', array( $this->mNewRev, &$revisionTools, $this->mOldRev ) );
 		$formattedRevisionTools = array();
 		// Put each one in parentheses (poor man's button)
-		foreach ( $revisionTools as $tool ) {
-			$formattedRevisionTools[] = $this->msg( 'parentheses' )->rawParams( $tool )->escaped();
+		foreach ( $revisionTools as $key => $tool ) {
+			$toolClass = is_string( $key ) ? $key : 'mw-diff-tool';
+			$element = Html::rawElement(
+				'span',
+				array( 'class' => $toolClass ),
+				$this->msg( 'parentheses' )->rawParams( $tool )->escaped()
+			);
+			$formattedRevisionTools[] = $element;
 		}
 		$newRevisionHeader = $this->getRevisionHeader( $this->mNewRev, 'complete' ) .
 			' ' . implode( ' ', $formattedRevisionTools );
@@ -445,7 +455,6 @@ class DifferenceEngine extends ContextSource {
 				$this->renderNewRevision();
 			}
 		}
-		wfProfileOut( __METHOD__ );
 	}
 
 	/**
@@ -482,7 +491,7 @@ class DifferenceEngine extends ContextSource {
 					array( 'USE INDEX' => 'rc_timestamp' )
 				);
 
-				if ( $change && $change->getPerformer()->getName() !== $user->getName() ) {
+				if ( $change && !$change->getPerformer()->equals( $user ) ) {
 					$rcid = $change->getAttribute( 'rc_id' );
 				} else {
 					// None found or the page has been created by the current user.
@@ -502,7 +511,7 @@ class DifferenceEngine extends ContextSource {
 					$this->mMarkPatrolledLink = ' <span class="patrollink">[' . Linker::linkKnown(
 						$this->mNewPage,
 						$this->msg( 'markaspatrolleddiff' )->escaped(),
-						array(),
+						array( 'class' => 'mw-patrollink' ),
 						array(
 							'action' => 'markpatrolled',
 							'rcid' => $rcid,
@@ -538,15 +547,14 @@ class DifferenceEngine extends ContextSource {
 	 * Show the new revision of the page.
 	 */
 	public function renderNewRevision() {
-		wfProfileIn( __METHOD__ );
 		$out = $this->getOutput();
 		$revHeader = $this->getRevisionHeader( $this->mNewRev );
 		# Add "current version as of X" title
-		$out->addHTML( "<hr class='diff-hr' />
+		$out->addHTML( "<hr class='diff-hr' id='mw-oldid' />
 		<h2 class='diff-currentversion-title'>{$revHeader}</h2>\n" );
 		# Page content may be handled by a hooked call instead...
 		# @codingStandardsIgnoreStart Ignoring long lines.
-		if ( wfRunHooks( 'ArticleContentOnDiff', array( $this, $out ) ) ) {
+		if ( Hooks::run( 'ArticleContentOnDiff', array( $this, $out ) ) ) {
 			$this->loadNewText();
 			$out->setRevisionId( $this->mNewid );
 			$out->setRevisionTimestamp( $this->mNewRev->getTimestamp() );
@@ -554,7 +562,7 @@ class DifferenceEngine extends ContextSource {
 
 			// NOTE: only needed for B/C: custom rendering of JS/CSS via hook
 			if ( $this->mNewPage->isCssJsSubpage() || $this->mNewPage->isCssOrJsPage() ) {
-				// Stolen from Article::view --AG 2007-10-11
+				// This needs to be synchronised with Article::showCssOrJsPage(), which sucks
 				// Give hooks a chance to customise the output
 				// @todo standardize this crap into one function
 				if ( ContentHandler::runLegacyHooks( 'ShowRawCssJs', array( $this->mNewContent, $this->mNewPage, $out ) ) ) {
@@ -562,10 +570,11 @@ class DifferenceEngine extends ContextSource {
 					// use the content object's own rendering
 					$cnt = $this->mNewRev->getContent();
 					$po = $cnt ? $cnt->getParserOutput( $this->mNewRev->getTitle(), $this->mNewRev->getId() ) : null;
-					$txt = $po ? $po->getText() : '';
-					$out->addHTML( $txt );
+					if ( $po ) {
+						$out->addParserOutputContent( $po );
+					}
 				}
-			} elseif ( !wfRunHooks( 'ArticleContentViewCustom', array( $this->mNewContent, $this->mNewPage, $out ) ) ) {
+			} elseif ( !Hooks::run( 'ArticleContentViewCustom', array( $this->mNewContent, $this->mNewPage, $out ) ) ) {
 				// Handled by extension
 			} elseif ( !ContentHandler::runLegacyHooks( 'ArticleViewCustom', array( $this->mNewContent, $this->mNewPage, $out ) ) ) {
 				// NOTE: deprecated hook, B/C only
@@ -584,19 +593,8 @@ class DifferenceEngine extends ContextSource {
 
 				$parserOutput = $this->getParserOutput( $wikiPage, $this->mNewRev );
 
-				# Also try to load it as a redirect
-				$rt = $this->mNewContent ? $this->mNewContent->getRedirectTarget() : null;
-
-				if ( $rt ) {
-					$article = Article::newFromTitle( $this->mNewPage, $this->getContext() );
-					$out->addHTML( $article->viewRedirect( $rt ) );
-
-					# WikiPage::getParserOutput() should not return false, but just in case
-					if ( $parserOutput ) {
-						# Show categories etc.
-						$out->addParserOutputNoText( $parserOutput );
-					}
-				} elseif ( $parserOutput ) {
+				# WikiPage::getParserOutput() should not return false, but just in case
+				if ( $parserOutput ) {
 					$out->addParserOutput( $parserOutput );
 				}
 			}
@@ -606,13 +604,12 @@ class DifferenceEngine extends ContextSource {
 		# Add redundant patrol link on bottom...
 		$out->addHTML( $this->markPatrolledLink() );
 
-		wfProfileOut( __METHOD__ );
 	}
 
 	protected function getParserOutput( WikiPage $page, Revision $rev ) {
 		$parserOptions = $page->makeParserOptions( $this->getContext() );
 
-		if ( !$rev->isCurrent() || !$rev->getTitle()->quickUserCan( "edit" ) ) {
+		if ( !$rev->isCurrent() || !$rev->getTitle()->quickUserCan( 'edit', $this->getUser() ) ) {
 			$parserOptions->setEditSection( false );
 		}
 
@@ -685,23 +682,19 @@ class DifferenceEngine extends ContextSource {
 	 */
 	public function getDiffBody() {
 		global $wgMemc;
-		wfProfileIn( __METHOD__ );
 		$this->mCacheHit = true;
 		// Check if the diff should be hidden from this user
 		if ( !$this->loadRevisionData() ) {
-			wfProfileOut( __METHOD__ );
 
 			return false;
 		} elseif ( $this->mOldRev &&
 			!$this->mOldRev->userCan( Revision::DELETED_TEXT, $this->getUser() )
 		) {
-			wfProfileOut( __METHOD__ );
 
 			return false;
 		} elseif ( $this->mNewRev &&
 			!$this->mNewRev->userCan( Revision::DELETED_TEXT, $this->getUser() )
 		) {
-			wfProfileOut( __METHOD__ );
 
 			return false;
 		}
@@ -709,7 +702,6 @@ class DifferenceEngine extends ContextSource {
 		if ( $this->mOldRev === false || ( $this->mOldRev && $this->mNewRev
 			&& $this->mOldRev->getID() == $this->mNewRev->getID() )
 		) {
-			wfProfileOut( __METHOD__ );
 
 			return '';
 		}
@@ -725,7 +717,6 @@ class DifferenceEngine extends ContextSource {
 					wfIncrStats( 'diff_cache_hit' );
 					$difftext = $this->localiseLineNumbers( $difftext );
 					$difftext .= "\n<!-- diff cache key $key -->\n";
-					wfProfileOut( __METHOD__ );
 
 					return $difftext;
 				}
@@ -735,7 +726,6 @@ class DifferenceEngine extends ContextSource {
 
 		// Loadtext is permission safe, this just clears out the diff
 		if ( !$this->loadText() ) {
-			wfProfileOut( __METHOD__ );
 
 			return false;
 		}
@@ -743,7 +733,7 @@ class DifferenceEngine extends ContextSource {
 		$difftext = $this->generateContentDiffBody( $this->mOldContent, $this->mNewContent );
 
 		// Save to cache for 7 days
-		if ( !wfRunHooks( 'AbortDiffCache', array( &$this ) ) ) {
+		if ( !Hooks::run( 'AbortDiffCache', array( &$this ) ) ) {
 			wfIncrStats( 'diff_uncacheable' );
 		} elseif ( $key !== false && $difftext !== false ) {
 			wfIncrStats( 'diff_cache_miss' );
@@ -755,7 +745,6 @@ class DifferenceEngine extends ContextSource {
 		if ( $difftext !== false ) {
 			$difftext = $this->localiseLineNumbers( $difftext );
 		}
-		wfProfileOut( __METHOD__ );
 
 		return $difftext;
 	}
@@ -833,15 +822,42 @@ class DifferenceEngine extends ContextSource {
 	 *
 	 * @todo move this to TextDifferenceEngine, make DifferenceEngine abstract. At some point.
 	 *
-	 * @param string $otext old text, must be already segmented
-	 * @param string $ntext new text, must be already segmented
+	 * @param string $otext Old text, must be already segmented
+	 * @param string $ntext New text, must be already segmented
 	 *
 	 * @return bool|string
 	 */
 	public function generateTextDiffBody( $otext, $ntext ) {
-		global $wgExternalDiffEngine, $wgContLang;
+		$self = $this;
+		$diff = function() use ( $self, $otext, $ntext ) {
+			return $self->textDiff( $otext, $ntext );
+		};
 
-		wfProfileIn( __METHOD__ );
+		$error = function( $status ) {
+			throw new FatalError( $status->getWikiText() );
+		};
+
+		// Use PoolCounter if the diff looks like it can be expensive
+		if ( strlen( $otext ) + strlen( $ntext ) > 20000 ) {
+			$work = new PoolCounterWorkViaCallback( 'diff',
+				md5( $otext ) . md5( $ntext ),
+				array( 'doWork' => $diff, 'error' => $error )
+			);
+			return $work->execute();
+		}
+
+		return $diff();
+	}
+
+	/**
+	 * Generates diff, to be wrapped internally in a logging/instrumentation
+	 *
+	 * @param string $otext Old text, must be already segmented
+	 * @param string $ntext New text, must be already segmented
+	 * @return bool|string
+	 */
+	public function textDiff( $otext, $ntext ) {
+		global $wgExternalDiffEngine, $wgContLang;
 
 		$otext = str_replace( "\r\n", "\n", $otext );
 		$ntext = str_replace( "\r\n", "\n", $ntext );
@@ -851,7 +867,6 @@ class DifferenceEngine extends ContextSource {
 			# input text to be HTML-escaped already
 			$otext = htmlspecialchars( $wgContLang->segmentForDiff( $otext ) );
 			$ntext = htmlspecialchars( $wgContLang->segmentForDiff( $ntext ) );
-			wfProfileOut( __METHOD__ );
 
 			return $wgContLang->unsegmentForDiff( wikidiff_do_diff( $otext, $ntext, 2 ) ) .
 			$this->debug( 'wikidiff1' );
@@ -860,11 +875,8 @@ class DifferenceEngine extends ContextSource {
 		if ( $wgExternalDiffEngine == 'wikidiff2' && function_exists( 'wikidiff2_do_diff' ) ) {
 			# Better external diff engine, the 2 may some day be dropped
 			# This one does the escaping and segmenting itself
-			wfProfileIn( 'wikidiff2_do_diff' );
 			$text = wikidiff2_do_diff( $otext, $ntext, 2 );
 			$text .= $this->debug( 'wikidiff2' );
-			wfProfileOut( 'wikidiff2_do_diff' );
-			wfProfileOut( __METHOD__ );
 
 			return $text;
 		}
@@ -876,13 +888,11 @@ class DifferenceEngine extends ContextSource {
 
 			$tempFile1 = fopen( $tempName1, "w" );
 			if ( !$tempFile1 ) {
-				wfProfileOut( __METHOD__ );
 
 				return false;
 			}
 			$tempFile2 = fopen( $tempName2, "w" );
 			if ( !$tempFile2 ) {
-				wfProfileOut( __METHOD__ );
 
 				return false;
 			}
@@ -891,13 +901,10 @@ class DifferenceEngine extends ContextSource {
 			fclose( $tempFile1 );
 			fclose( $tempFile2 );
 			$cmd = wfEscapeShellArg( $wgExternalDiffEngine, $tempName1, $tempName2 );
-			wfProfileIn( __METHOD__ . "-shellexec" );
 			$difftext = wfShellExec( $cmd );
 			$difftext .= $this->debug( "external $wgExternalDiffEngine" );
-			wfProfileOut( __METHOD__ . "-shellexec" );
 			unlink( $tempName1 );
 			unlink( $tempName2 );
-			wfProfileOut( __METHOD__ );
 
 			return $difftext;
 		}
@@ -907,8 +914,7 @@ class DifferenceEngine extends ContextSource {
 		$nta = explode( "\n", $wgContLang->segmentForDiff( $ntext ) );
 		$diffs = new Diff( $ota, $nta );
 		$formatter = new TableDiffFormatter();
-		$difftext = $wgContLang->unsegmentForDiff( $formatter->format( $diffs ) ) .
-			wfProfileOut( __METHOD__ );
+		$difftext = $wgContLang->unsegmentForDiff( $formatter->format( $diffs ) );
 
 		return $difftext;
 	}
@@ -982,14 +988,14 @@ class DifferenceEngine extends ContextSource {
 		}
 
 		// Sanity: don't show the notice if too many rows must be scanned
-		// @TODO: show some special message for that case
+		// @todo show some special message for that case
 		$nEdits = $this->mNewPage->countRevisionsBetween( $oldRev, $newRev, 1000 );
 		if ( $nEdits > 0 && $nEdits <= 1000 ) {
 			$limit = 100; // use diff-multi-manyusers if too many users
 			$users = $this->mNewPage->getAuthorsBetween( $oldRev, $newRev, $limit );
 			$numUsers = count( $users );
 
-			if ( $numUsers == 1 && $users[0] == $newRev->getRawUserText() ) {
+			if ( $numUsers == 1 && $users[0] == $newRev->getUserText( Revision::RAW ) ) {
 				$numUsers = 0; // special case to say "by the same user" instead of "by one other user"
 			}
 
@@ -1062,8 +1068,13 @@ class DifferenceEngine extends ContextSource {
 
 			$key = $title->quickUserCan( 'edit', $user ) ? 'editold' : 'viewsourceold';
 			$msg = $this->msg( $key )->escaped();
-			$header .= ' ' . $this->msg( 'parentheses' )->rawParams(
-				Linker::linkKnown( $title, $msg, array(), $editQuery ) )->plain();
+			$editLink = $this->msg( 'parentheses' )->rawParams(
+				Linker::linkKnown( $title, $msg, array( ), $editQuery ) )->escaped();
+			$header .= ' ' . Html::rawElement(
+				'span',
+				array( 'class' => 'mw-diff-edit' ),
+				$editLink
+			);
 			if ( $rev->isDeleted( Revision::DELETED_TEXT ) ) {
 				$header = Html::rawElement(
 					'span',
@@ -1150,6 +1161,8 @@ class DifferenceEngine extends ContextSource {
 
 	/**
 	 * Use specified text instead of loading from the database
+	 * @param Content $oldContent
+	 * @param Content $newContent
 	 * @since 1.21
 	 */
 	public function setContent( Content $oldContent, Content $newContent ) {
@@ -1163,6 +1176,7 @@ class DifferenceEngine extends ContextSource {
 	/**
 	 * Set the language in which the diff text is written
 	 * (Defaults to page content language).
+	 * @param Language|string $lang
 	 * @since 1.19
 	 */
 	public function setTextLanguage( $lang ) {
@@ -1217,7 +1231,7 @@ class DifferenceEngine extends ContextSource {
 			$this->mNewid = 0;
 		}
 
-		wfRunHooks(
+		Hooks::run(
 			'NewDifferenceEngine',
 			array( $this->getTitle(), &$this->mOldid, &$this->mNewid, $old, $new )
 		);
