@@ -47,7 +47,7 @@ class MysqlInstaller extends DatabaseInstaller {
 
 	public $supportedEngines = array( 'InnoDB', 'MyISAM' );
 
-	public $minimumVersion = '5.0.2';
+	public $minimumVersion = '5.0.3';
 
 	public $webUserPrivs = array(
 		'DELETE',
@@ -65,19 +65,12 @@ class MysqlInstaller extends DatabaseInstaller {
 	}
 
 	/**
-	 * @return Bool
+	 * @return bool
 	 */
 	public function isCompiled() {
 		return self::checkExtension( 'mysql' ) || self::checkExtension( 'mysqli' );
 	}
 
-	/**
-	 * @return array
-	 */
-	public function getGlobalDefaults() {
-		return array();
-	}
-
 	/**
 	 * @return string
 	 */
@@ -120,7 +113,7 @@ class MysqlInstaller extends DatabaseInstaller {
 		if ( !strlen( $newValues['_InstallUser'] ) ) {
 			$status->fatal( 'config-db-username-empty' );
 		}
-		if (!strlen( $newValues['_InstallPassword'] ) ) {
+		if ( !strlen( $newValues['_InstallPassword'] ) ) {
 			$status->fatal( 'config-db-password-empty', $newValues['_InstallUser'] );
 		}
 		if ( !$status->isOK() ) {
@@ -603,7 +596,7 @@ class MysqlInstaller extends DatabaseInstaller {
 	 * Return a formal 'User'@'Host' username for use in queries
 	 * @param string $name Username, quotes will be added
 	 * @param string $host Hostname, quotes will be added
-	 * @return String
+	 * @return string
 	 */
 	private function buildFullUserName( $name, $host ) {
 		return $this->db->addQuotes( $name ) . '@' . $this->db->addQuotes( $host );
@@ -614,7 +607,7 @@ class MysqlInstaller extends DatabaseInstaller {
 	 * access to mysql.user, so false means "no" or "maybe"
 	 * @param string $host Hostname to check
 	 * @param string $user Username to check
-	 * @return boolean
+	 * @return bool
 	 */
 	private function userDefinitelyExists( $host, $user ) {
 		try {
@@ -631,7 +624,7 @@ class MysqlInstaller extends DatabaseInstaller {
 	 * Return any table options to be applied to all tables that don't
 	 * override them.
 	 *
-	 * @return String
+	 * @return string
 	 */
 	protected function getTableOptions() {
 		$options = array();
