@@ -27,7 +27,6 @@
  * of a specific loader request
  */
 class ResourceLoaderContext {
-
 	/* Protected Members */
 
 	protected $resourceLoader;
@@ -42,16 +41,19 @@ class ResourceLoaderContext {
 	protected $version;
 	protected $hash;
 	protected $raw;
+	protected $image;
+	protected $variant;
+	protected $format;
+	protected $userObj;
+	protected $imageObj;
 
 	/* Methods */
 
 	/**
-	 * @param $resourceLoader ResourceLoader
-	 * @param $request WebRequest
+	 * @param ResourceLoader $resourceLoader
+	 * @param WebRequest $request
 	 */
-	public function __construct( $resourceLoader, WebRequest $request ) {
-		global $wgDefaultSkin, $wgResourceLoaderDebug;
-
+	public function __construct( ResourceLoader $resourceLoader, WebRequest $request ) {
 		$this->resourceLoader = $resourceLoader;
 		$this->request = $request;
 
@@ -62,15 +64,21 @@ class ResourceLoaderContext {
 		// Various parameters
 		$this->skin = $request->getVal( 'skin' );
 		$this->user = $request->getVal( 'user' );
-		$this->debug = $request->getFuzzyBool( 'debug', $wgResourceLoaderDebug );
+		$this->debug = $request->getFuzzyBool(
+			'debug', $resourceLoader->getConfig()->get( 'ResourceLoaderDebug' )
+		);
 		$this->only = $request->getVal( 'only' );
 		$this->version = $request->getVal( 'version' );
 		$this->raw = $request->getFuzzyBool( 'raw' );
+		// Image requests
+		$this->image = $request->getVal( 'image' );
+		$this->variant = $request->getVal( 'variant' );
+		$this->format = $request->getVal( 'format' );
 
 		$skinnames = Skin::getSkinNames();
 		// If no skin is specified, or we don't recognize the skin, use the default skin
 		if ( !$this->skin || !isset( $skinnames[$this->skin] ) ) {
-			$this->skin = $wgDefaultSkin;
+			$this->skin = $resourceLoader->getConfig()->get( 'DefaultSkin' );
 		}
 	}
 
@@ -79,7 +87,7 @@ class ResourceLoaderContext {
 	 * an array of module names like array( 'jquery.foo', 'jquery.bar',
 	 * 'jquery.ui.baz', 'jquery.ui.quux' )
 	 * @param string $modules Packed module name list
-	 * @return array of module names
+	 * @return array Array of module names
 	 */
 	public static function expandModuleNames( $modules ) {
 		$retval = array();
@@ -109,11 +117,14 @@ class ResourceLoaderContext {
 	}
 
 	/**
-	 * Return a dummy ResourceLoaderContext object suitable for passing into things that don't "really" need a context
+	 * Return a dummy ResourceLoaderContext object suitable for passing into
+	 * things that don't "really" need a context.
 	 * @return ResourceLoaderContext
 	 */
 	public static function newDummyContext() {
-		return new self( null, new FauxRequest( array() ) );
+		return new self( new ResourceLoader(
+			ConfigFactory::getDefaultInstance()->makeConfig( 'main' )
+		), new FauxRequest( array() ) );
 	}
 
 	/**
@@ -176,6 +187,31 @@ class ResourceLoaderContext {
 		return $this->user;
 	}
 
+	/**
+	 * Get the possibly-cached User object for the specified username
+	 *
+	 * @since 1.25
+	 * @return User|bool false if a valid object cannot be created
+	 */
+	public function getUserObj() {
+		if ( $this->userObj === null ) {
+			$username = $this->getUser();
+			if ( $username ) {
+				// Optimize: Avoid loading a new User object if possible
+				global $wgUser;
+				if ( is_object( $wgUser ) && $wgUser->getName() === $username ) {
+					$this->userObj = $wgUser;
+				} else {
+					$this->userObj = User::newFromName( $username );
+				}
+			} else {
+				$this->userObj = new User; // Anonymous user
+			}
+		}
+
+		return $this->userObj;
+	}
+
 	/**
 	 * @return bool
 	 */
@@ -184,14 +220,14 @@ class ResourceLoaderContext {
 	}
 
 	/**
-	 * @return String|null
+	 * @return string|null
 	 */
 	public function getOnly() {
 		return $this->only;
 	}
 
 	/**
-	 * @return String|null
+	 * @return string|null
 	 */
 	public function getVersion() {
 		return $this->version;
@@ -204,25 +240,81 @@ class ResourceLoaderContext {
 		return $this->raw;
 	}
 
+	/**
+	 * @return string|null
+	 */
+	public function getImage() {
+		return $this->image;
+	}
+
+	/**
+	 * @return string|null
+	 */
+	public function getVariant() {
+		return $this->variant;
+	}
+
+	/**
+	 * @return string|null
+	 */
+	public function getFormat() {
+		return $this->format;
+	}
+
+	/**
+	 * If this is a request for an image, get the ResourceLoaderImage object.
+	 *
+	 * @since 1.25
+	 * @return ResourceLoaderImage|bool false if a valid object cannot be created
+	 */
+	public function getImageObj() {
+		if ( $this->imageObj === null ) {
+			$this->imageObj = false;
+
+			if ( !$this->image ) {
+				return $this->imageObj;
+			}
+
+			$modules = $this->getModules();
+			if ( count( $modules ) !== 1 ) {
+				return $this->imageObj;
+			}
+
+			$module = $this->getResourceLoader()->getModule( $modules[0] );
+			if ( !$module || !$module instanceof ResourceLoaderImageModule ) {
+				return $this->imageObj;
+			}
+
+			$image = $module->getImage( $this->image );
+			if ( !$image ) {
+				return $this->imageObj;
+			}
+
+			$this->imageObj = $image;
+		}
+
+		return $this->imageObj;
+	}
+
 	/**
 	 * @return bool
 	 */
 	public function shouldIncludeScripts() {
-		return is_null( $this->only ) || $this->only === 'scripts';
+		return is_null( $this->getOnly() ) || $this->getOnly() === 'scripts';
 	}
 
 	/**
 	 * @return bool
 	 */
 	public function shouldIncludeStyles() {
-		return is_null( $this->only ) || $this->only === 'styles';
+		return is_null( $this->getOnly() ) || $this->getOnly() === 'styles';
 	}
 
 	/**
 	 * @return bool
 	 */
 	public function shouldIncludeMessages() {
-		return is_null( $this->only ) || $this->only === 'messages';
+		return is_null( $this->getOnly() ) || $this->getOnly() === 'messages';
 	}
 
 	/**
@@ -231,8 +323,9 @@ class ResourceLoaderContext {
 	public function getHash() {
 		if ( !isset( $this->hash ) ) {
 			$this->hash = implode( '|', array(
-				$this->getLanguage(), $this->getDirection(), $this->skin, $this->user,
-				$this->debug, $this->only, $this->version
+				$this->getLanguage(), $this->getDirection(), $this->getSkin(), $this->getUser(),
+				$this->getImage(), $this->getVariant(), $this->getFormat(),
+				$this->getDebug(), $this->getOnly(), $this->getVersion()
 			) );
 		}
 		return $this->hash;
