@@ -27,7 +27,6 @@
  * Includes some static functions for handling the special page list deprecated
  * in favor of SpecialPageFactory.
  *
- * @todo Turn this into a real ContextSource
  * @ingroup SpecialPage
  */
 class SpecialPage {
@@ -40,17 +39,11 @@ class SpecialPage {
 
 	// Minimum user level required to access this page, or "" for anyone.
 	// Also used to categorise the pages in Special:Specialpages
-	private $mRestriction;
+	protected $mRestriction;
 
 	// Listed in Special:Specialpages?
 	private $mListed;
 
-	// Function name called by the default execute()
-	private $mFunction;
-
-	// File which needs to be included before the function above can be called
-	private $mFile;
-
 	// Whether or not this special page is being included from an article
 	protected $mIncluding;
 
@@ -107,30 +100,18 @@ class SpecialPage {
 	 * @param string $name Name of the special page, as seen in links and URLs
 	 * @param string $restriction User right required, e.g. "block" or "delete"
 	 * @param bool $listed Whether the page is listed in Special:Specialpages
-	 * @param callable|bool $function Function called by execute(). By default
-	 *   it is constructed from $name
-	 * @param string $file File which is included by execute(). It is also
-	 *   constructed from $name by default
+	 * @param callable|bool $function Unused
+	 * @param string $file Unused
 	 * @param bool $includable Whether the page can be included in normal pages
 	 */
 	public function __construct(
 		$name = '', $restriction = '', $listed = true,
-		$function = false, $file = 'default', $includable = false
+		$function = false, $file = '', $includable = false
 	) {
 		$this->mName = $name;
 		$this->mRestriction = $restriction;
 		$this->mListed = $listed;
 		$this->mIncludable = $includable;
-		if ( !$function ) {
-			$this->mFunction = 'wfSpecial' . $name;
-		} else {
-			$this->mFunction = $function;
-		}
-		if ( $file === 'default' ) {
-			$this->mFile = __DIR__ . "/specials/Special$name.php";
-		} else {
-			$this->mFile = $file;
-		}
 	}
 
 	/**
@@ -149,23 +130,10 @@ class SpecialPage {
 		return $this->mRestriction;
 	}
 
-	/**
-	 * Get the file which will be included by SpecialPage::execute() if your extension is
-	 * still stuck in the past and hasn't overridden the execute() method.  No modern code
-	 * should want or need to know this.
-	 * @return string
-	 * @deprecated since 1.18
-	 */
-	function getFile() {
-		wfDeprecated( __METHOD__, '1.18' );
-
-		return $this->mFile;
-	}
-
 	// @todo FIXME: Decide which syntax to use for this, and stick to it
 	/**
 	 * Whether this special page is listed in Special:SpecialPages
-	 * @since r3583 (v1.3)
+	 * @since 1.3 (r3583)
 	 * @return bool
 	 */
 	function isListed() {
@@ -305,48 +273,78 @@ class SpecialPage {
 	}
 
 	/**
-	 * If the user is not logged in, throws UserNotLoggedIn error.
+	 * If the user is not logged in, throws UserNotLoggedIn error
 	 *
-	 * Default error message includes a link to Special:Userlogin with properly set 'returnto' query
-	 * parameter.
+	 * The user will be redirected to Special:Userlogin with the given message as an error on
+	 * the form.
 	 *
 	 * @since 1.23
-	 * @param string|Message $reasonMsg [optional] Passed on to UserNotLoggedIn constructor. Strings
-	 *     will be used as message keys. If a string is given, the message will also receive a
-	 *     formatted login link (generated using the 'loginreqlink' message) as first parameter. If a
-	 *     Message is given, it will be passed on verbatim.
-	 * @param string|Message $titleMsg [optional] Passed on to UserNotLoggedIn constructor. Strings
-	 *     will be used as message keys.
+	 * @param string $reasonMsg [optional] Message key to be displayed on login page
+	 * @param string $titleMsg [optional] Passed on to UserNotLoggedIn constructor
 	 * @throws UserNotLoggedIn
 	 */
-	public function requireLogin( $reasonMsg = null, $titleMsg = null ) {
+	public function requireLogin(
+		$reasonMsg = 'exception-nologin-text', $titleMsg = 'exception-nologin'
+	) {
 		if ( $this->getUser()->isAnon() ) {
-			// Use default messages if not given or explicit null passed
-			if ( !$reasonMsg ) {
-				$reasonMsg = 'exception-nologin-text-manual';
-			}
-			if ( !$titleMsg ) {
-				$titleMsg = 'exception-nologin';
-			}
-
-			// Convert to Messages with current context
-			if ( is_string( $reasonMsg ) ) {
-				$loginreqlink = Linker::linkKnown(
-					SpecialPage::getTitleFor( 'Userlogin' ),
-					$this->msg( 'loginreqlink' )->escaped(),
-					array(),
-					array( 'returnto' => $this->getPageTitle()->getPrefixedText() )
-				);
-				$reasonMsg = $this->msg( $reasonMsg )->rawParams( $loginreqlink );
-			}
-			if ( is_string( $titleMsg ) ) {
-				$titleMsg = $this->msg( $titleMsg );
-			}
-
 			throw new UserNotLoggedIn( $reasonMsg, $titleMsg );
 		}
 	}
 
+	/**
+	 * Return an array of subpages beginning with $search that this special page will accept.
+	 *
+	 * For example, if a page supports subpages "foo", "bar" and "baz" (as in Special:PageName/foo,
+	 * etc.):
+	 *
+	 *   - `prefixSearchSubpages( "ba" )` should return `array( "bar", "baz" )`
+	 *   - `prefixSearchSubpages( "f" )` should return `array( "foo" )`
+	 *   - `prefixSearchSubpages( "z" )` should return `array()`
+	 *   - `prefixSearchSubpages( "" )` should return `array( foo", "bar", "baz" )`
+	 *
+	 * @param string $search Prefix to search for
+	 * @param int $limit Maximum number of results to return (usually 10)
+	 * @param int $offset Number of results to skip (usually 0)
+	 * @return string[] Matching subpages
+	 */
+	public function prefixSearchSubpages( $search, $limit, $offset ) {
+		$subpages = $this->getSubpagesForPrefixSearch();
+		if ( !$subpages ) {
+			return array();
+		}
+
+		return self::prefixSearchArray( $search, $limit, $subpages, $offset );
+	}
+
+	/**
+	 * Return an array of subpages that this special page will accept for prefix
+	 * searches. If this method requires a query you might instead want to implement
+	 * prefixSearchSubpages() directly so you can support $limit and $offset. This
+	 * method is better for static-ish lists of things.
+	 *
+	 * @return string[] subpages to search from
+	 */
+	protected function getSubpagesForPrefixSearch() {
+		return array();
+	}
+
+	/**
+	 * Helper function for implementations of prefixSearchSubpages() that
+	 * filter the values in memory (as opposed to making a query).
+	 *
+	 * @since 1.24
+	 * @param string $search
+	 * @param int $limit
+	 * @param array $subpages
+	 * @param int $offset
+	 * @return string[]
+	 */
+	protected static function prefixSearchArray( $search, $limit, array $subpages, $offset ) {
+		$escaped = preg_quote( $search, '/' );
+		return array_slice( preg_grep( "/^$escaped/i",
+			array_slice( $subpages, $offset ) ), 0, $limit );
+	}
+
 	/**
 	 * Sets headers - this should be called from the execute() method of all derived classes!
 	 */
@@ -355,6 +353,13 @@ class SpecialPage {
 		$out->setArticleRelated( false );
 		$out->setRobotPolicy( $this->getRobotPolicy() );
 		$out->setPageTitle( $this->getDescription() );
+		if ( $this->getConfig()->get( 'UseMediaWikiUIEverywhere' ) ) {
+			$out->addModuleStyles( array(
+				'mediawiki.ui.input',
+				'mediawiki.ui.radio',
+				'mediawiki.ui.checkbox',
+			) );
+		}
 	}
 
 	/**
@@ -373,7 +378,7 @@ class SpecialPage {
 		 * @param SpecialPage $this
 		 * @param string|null $subPage
 		 */
-		wfRunHooks( 'SpecialPageBeforeExecute', array( $this, $subPage ) );
+		Hooks::run( 'SpecialPageBeforeExecute', array( $this, $subPage ) );
 
 		$this->beforeExecute( $subPage );
 		$this->execute( $subPage );
@@ -387,7 +392,7 @@ class SpecialPage {
 		 * @param SpecialPage $this
 		 * @param string|null $subPage
 		 */
-		wfRunHooks( 'SpecialPageAfterExecute', array( $this, $subPage ) );
+		Hooks::run( 'SpecialPageAfterExecute', array( $this, $subPage ) );
 	}
 
 	/**
@@ -414,7 +419,7 @@ class SpecialPage {
 
 	/**
 	 * Default execute method
-	 * Checks user permissions, calls the function given in mFunction
+	 * Checks user permissions
 	 *
 	 * This must be overridden by subclasses; it will be made abstract in a future version
 	 *
@@ -423,14 +428,7 @@ class SpecialPage {
 	public function execute( $subPage ) {
 		$this->setHeaders();
 		$this->checkPermissions();
-
-		$func = $this->mFunction;
-		// only load file if the function does not exist
-		if ( !is_callable( $func ) && $this->mFile ) {
-			require_once $this->mFile;
-		}
 		$this->outputHeader();
-		call_user_func( $func, $subPage, $this );
 	}
 
 	/**
@@ -473,7 +471,7 @@ class SpecialPage {
 	 *
 	 * @param string|bool $subpage
 	 * @return Title
-	 * @deprecated in 1.23, use SpecialPage::getPageTitle
+	 * @deprecated since 1.23, use SpecialPage::getPageTitle
 	 */
 	function getTitle( $subpage = false ) {
 		return $this->getPageTitle( $subpage );
@@ -557,19 +555,6 @@ class SpecialPage {
 		return $this->getContext()->getSkin();
 	}
 
-	/**
-	 * Shortcut to get user's language
-	 *
-	 * @deprecated since 1.19 Use getLanguage instead
-	 * @return Language
-	 * @since 1.18
-	 */
-	public function getLang() {
-		wfDeprecated( __METHOD__, '1.19' );
-
-		return $this->getLanguage();
-	}
-
 	/**
 	 * Shortcut to get user's language
 	 *
@@ -580,6 +565,15 @@ class SpecialPage {
 		return $this->getContext()->getLanguage();
 	}
 
+	/**
+	 * Shortcut to get main config object
+	 * @return Config
+	 * @since 1.24
+	 */
+	public function getConfig() {
+		return $this->getContext()->getConfig();
+	}
+
 	/**
 	 * Return the full title, including $par
 	 *
@@ -629,17 +623,35 @@ class SpecialPage {
 	 * @param array $params
 	 */
 	protected function addFeedLinks( $params ) {
-		global $wgFeedClasses;
-
 		$feedTemplate = wfScript( 'api' );
 
-		foreach ( $wgFeedClasses as $format => $class ) {
+		foreach ( $this->getConfig()->get( 'FeedClasses' ) as $format => $class ) {
 			$theseParams = $params + array( 'feedformat' => $format );
 			$url = wfAppendQuery( $feedTemplate, $theseParams );
 			$this->getOutput()->addFeedLink( $format, $url );
 		}
 	}
 
+	/**
+	 * Adds help link with an icon via page indicators.
+	 * Link target can be overridden by a local message containing a wikilink:
+	 * the message key is: lowercase special page name + '-helppage'.
+	 * @param string $to Target MediaWiki.org page title or encoded URL.
+	 * @param bool $overrideBaseUrl Whether $url is a full URL, to avoid MW.o.
+	 * @since 1.25
+	 */
+	public function addHelpLink( $to, $overrideBaseUrl = false ) {
+		global $wgContLang;
+		$msg = $this->msg( $wgContLang->lc( $this->getName() ) . '-helppage' );
+
+		if ( !$msg->isDisabled() ) {
+			$helpUrl = Skin::makeUrl( $msg->plain() );
+			$this->getOutput()->addHelpLink( $helpUrl, true );
+		} else {
+			$this->getOutput()->addHelpLink( $to, $overrideBaseUrl );
+		}
+	}
+
 	/**
 	 * Get the group that the special page belongs in on Special:SpecialPage
 	 * Use this method, instead of getGroupName to allow customization
@@ -649,8 +661,8 @@ class SpecialPage {
 	 * @since 1.21
 	 */
 	public function getFinalGroupName() {
-		global $wgSpecialPageGroups;
 		$name = $this->getName();
+		$specialPageGroups = $this->getConfig()->get( 'SpecialPageGroups' );
 
 		// Allow overbidding the group from the wiki side
 		$msg = $this->msg( 'specialpages-specialpagegroup-' . strtolower( $name ) )->inContentLanguage();
@@ -663,8 +675,8 @@ class SpecialPage {
 			// Group '-' is used as default to have the chance to determine,
 			// if the special pages overrides this method,
 			// if not overridden, $wgSpecialPageGroups is checked for b/c
-			if ( $group === '-' && isset( $wgSpecialPageGroups[$name] ) ) {
-				$group = $wgSpecialPageGroups[$name];
+			if ( $group === '-' && isset( $specialPageGroups[$name] ) ) {
+				$group = $specialPageGroups[$name];
 			}
 		}
 
