@@ -27,6 +27,8 @@
  * @ingroup SpecialPage
  */
 class LinkSearchPage extends QueryPage {
+	/** @var array|bool */
+	private $mungedQuery = false;
 
 	/**
 	 * @var PageLinkRenderer
@@ -51,7 +53,7 @@ class LinkSearchPage extends QueryPage {
 	 * Initialize or override the PageLinkRenderer LinkSearchPage collaborates with.
 	 * Useful mainly for testing.
 	 *
-	 * @todo: query logic and rendering logic should be split and also injected
+	 * @todo query logic and rendering logic should be split and also injected
 	 *
 	 * @param PageLinkRenderer $linkRenderer
 	 */
@@ -66,8 +68,9 @@ class LinkSearchPage extends QueryPage {
 	 * This allows for dependency injection even though we don't control object creation.
 	 */
 	private function initServices() {
+		global $wgLanguageCode;
 		if ( !$this->linkRenderer ) {
-			$lang = $this->getContext()->getLanguage();
+			$lang = Language::factory( $wgLanguageCode );
 			$titleFormatter = new MediaWikiTitleCodec( $lang, GenderCache::singleton() );
 			$this->linkRenderer = new MediaWikiPageLinkRenderer( $titleFormatter );
 		}
@@ -78,8 +81,6 @@ class LinkSearchPage extends QueryPage {
 	}
 
 	function execute( $par ) {
-		global $wgUrlProtocols, $wgMiserMode, $wgScript;
-
 		$this->initServices();
 
 		$this->setHeaders();
@@ -90,10 +91,10 @@ class LinkSearchPage extends QueryPage {
 
 		$request = $this->getRequest();
 		$target = $request->getVal( 'target', $par );
-		$namespace = $request->getIntorNull( 'namespace', null );
+		$namespace = $request->getIntOrNull( 'namespace', null );
 
 		$protocols_list = array();
-		foreach ( $wgUrlProtocols as $prot ) {
+		foreach ( $this->getConfig()->get( 'UrlProtocols' ) as $prot ) {
 			if ( $prot !== '//' ) {
 				$protocols_list[] = $prot;
 			}
@@ -122,7 +123,7 @@ class LinkSearchPage extends QueryPage {
 		);
 		$s = Html::openElement(
 			'form',
-			array( 'id' => 'mw-linksearch-form', 'method' => 'get', 'action' => $wgScript )
+			array( 'id' => 'mw-linksearch-form', 'method' => 'get', 'action' => wfScript() )
 		) . "\n" .
 			Html::hidden( 'title', $this->getPageTitle()->getPrefixedDBkey() ) . "\n" .
 			Html::openElement( 'fieldset' ) . "\n" .
@@ -132,10 +133,14 @@ class LinkSearchPage extends QueryPage {
 				'target',
 				'target',
 				50,
-				$target
+				$target,
+				array(
+					// URLs are always ltr
+					'dir' => 'ltr',
+				)
 			) . "\n";
 
-		if ( !$wgMiserMode ) {
+		if ( !$this->getConfig()->get( 'MiserMode' ) ) {
 			$s .= Html::namespaceSelector(
 				array(
 					'selected' => $namespace,
@@ -160,7 +165,7 @@ class LinkSearchPage extends QueryPage {
 				'namespace' => $namespace,
 				'protocol' => $protocol ) );
 			parent::execute( $par );
-			if ( $this->mMungedQuery === false ) {
+			if ( $this->mungedQuery === false ) {
 				$out->addWikiMsg( 'linksearch-error' );
 			}
 		}
@@ -177,8 +182,8 @@ class LinkSearchPage extends QueryPage {
 	/**
 	 * Return an appropriately formatted LIKE query and the clause
 	 *
-	 * @param String $query Search pattern to search for
-	 * @param String $prot Protocol, e.g. 'http://'
+	 * @param string $query Search pattern to search for
+	 * @param string $prot Protocol, e.g. 'http://'
 	 *
 	 * @return array
 	 */
@@ -206,10 +211,9 @@ class LinkSearchPage extends QueryPage {
 	}
 
 	function linkParameters() {
-		global $wgMiserMode;
 		$params = array();
 		$params['target'] = $this->mProt . $this->mQuery;
-		if ( isset( $this->mNs ) && !$wgMiserMode ) {
+		if ( $this->mNs !== null && !$this->getConfig()->get( 'MiserMode' ) ) {
 			$params['namespace'] = $this->mNs;
 		}
 
@@ -217,17 +221,16 @@ class LinkSearchPage extends QueryPage {
 	}
 
 	function getQueryInfo() {
-		global $wgMiserMode;
 		$dbr = wfGetDB( DB_SLAVE );
 		// strip everything past first wildcard, so that
 		// index-based-only lookup would be done
-		list( $this->mMungedQuery, $clause ) = self::mungeQuery( $this->mQuery, $this->mProt );
-		if ( $this->mMungedQuery === false ) {
+		list( $this->mungedQuery, $clause ) = self::mungeQuery( $this->mQuery, $this->mProt );
+		if ( $this->mungedQuery === false ) {
 			// Invalid query; return no results
 			return array( 'tables' => 'page', 'fields' => 'page_id', 'conds' => '0=1' );
 		}
 
-		$stripped = LinkFilter::keepOneWildcard( $this->mMungedQuery );
+		$stripped = LinkFilter::keepOneWildcard( $this->mungedQuery );
 		$like = $dbr->buildLike( $stripped );
 		$retval = array(
 			'tables' => array( 'page', 'externallinks' ),
@@ -244,13 +247,32 @@ class LinkSearchPage extends QueryPage {
 			'options' => array( 'USE INDEX' => $clause )
 		);
 
-		if ( isset( $this->mNs ) && !$wgMiserMode ) {
+		if ( $this->mNs !== null && !$this->getConfig()->get( 'MiserMode' ) ) {
 			$retval['conds']['page_namespace'] = $this->mNs;
 		}
 
 		return $retval;
 	}
 
+	/**
+	 * Pre-fill the link cache
+	 *
+	 * @param IDatabase $db
+	 * @param ResultWrapper $res
+	 */
+	function preprocessResults( $db, $res ) {
+		if ( $res->numRows() > 0 ) {
+			$linkBatch = new LinkBatch();
+
+			foreach ( $res as $row ) {
+				$linkBatch->add( $row->namespace, $row->title );
+			}
+
+			$res->seek( 0 );
+			$linkBatch->execute();
+		}
+	}
+
 	/**
 	 * @param Skin $skin
 	 * @param object $result Result row
@@ -266,24 +288,6 @@ class LinkSearchPage extends QueryPage {
 		return $this->msg( 'linksearch-line' )->rawParams( $urlLink, $pageLink )->escaped();
 	}
 
-	/**
-	 * Override to check query validity.
-	 *
-	 * @param mixed $offset Numerical offset or false for no offset
-	 * @param mixed $limit Numerical limit or false for no limit
-	 */
-	function doQuery( $offset = false, $limit = false ) {
-		list( $this->mMungedQuery, ) = LinkSearchPage::mungeQuery( $this->mQuery, $this->mProt );
-		if ( $this->mMungedQuery === false ) {
-			$this->getOutput()->addWikiMsg( 'linksearch-error' );
-		} else {
-			// For debugging
-			// Generates invalid xhtml with patterns that contain --
-			//$this->getOutput()->addHTML( "\n<!-- " . htmlspecialchars( $this->mMungedQuery ) . " -->\n" );
-			parent::doQuery( $offset, $limit );
-		}
-	}
-
 	/**
 	 * Override to squash the ORDER BY.
 	 * We do a truncated index search, so the optimizer won't trust
