@@ -55,6 +55,8 @@ class SpecialPreferences extends SpecialPage {
 			);
 		}
 
+		$this->addHelpLink( 'Help:Preferences' );
+
 		$htmlForm = Preferences::getFormObject( $this->getUser(), $this->getContext() );
 		$htmlForm->setSubmitCallback( array( 'Preferences', 'tryUISubmit' ) );
 
@@ -73,6 +75,7 @@ class SpecialPreferences extends SpecialPage {
 		$htmlForm = new HTMLForm( array(), $context, 'prefs-restore' );
 
 		$htmlForm->setSubmitTextMsg( 'restoreprefs' );
+		$htmlForm->setSubmitDestructive();
 		$htmlForm->setSubmitCallback( array( $this, 'submitReset' ) );
 		$htmlForm->suppressReset();
 
