@@ -41,6 +41,29 @@ class LoginForm extends SpecialPage {
 	const USER_BLOCKED = 11;
 	const NEED_TOKEN = 12;
 	const WRONG_TOKEN = 13;
+	const USER_MIGRATED = 14;
+
+	/**
+	 * Valid error and warning messages
+	 *
+	 * Special:Userlogin can show an error or warning message on the form when
+	 * coming from another page. This is done via the ?error= or ?warning= GET
+	 * parameters.
+	 *
+	 * This array is the list of valid message keys. All other values will be
+	 * ignored.
+	 *
+	 * @since 1.24
+	 * @var string[]
+	 */
+	public static $validErrorMessages = array(
+		'exception-nologin-text',
+		'watchlistanontext',
+		'changeemail-no-info',
+		'resetpass-no-info',
+		'confirmemail_needlogin',
+		'prefsnologintext2',
+	);
 
 	public $mAbortLoginErrorMsg = null;
 
@@ -65,6 +88,8 @@ class LoginForm extends SpecialPage {
 	protected $mType;
 	protected $mReason;
 	protected $mRealName;
+	protected $mEntryError = '';
+	protected $mEntryErrorType = 'error';
 
 	private $mTempPasswordUsed;
 	private $mLoaded = false;
@@ -80,9 +105,27 @@ class LoginForm extends SpecialPage {
 	 * @param WebRequest $request
 	 */
 	public function __construct( $request = null ) {
+		global $wgUseMediaWikiUIEverywhere;
 		parent::__construct( 'Userlogin' );
 
 		$this->mOverrideRequest = $request;
+		// Override UseMediaWikiEverywhere to true, to force login and create form to use mw ui
+		$wgUseMediaWikiUIEverywhere = true;
+	}
+
+	/**
+	 * Returns an array of all valid error messages.
+	 *
+	 * @return array
+	 */
+	public static function getValidErrorMessages() {
+		static $messages = null;
+		if ( !$messages ) {
+			$messages = self::$validErrorMessages;
+			Hooks::run( 'LoginFormValidErrorMessages', array( &$messages ) );
+		}
+
+		return $messages;
 	}
 
 	/**
@@ -117,7 +160,8 @@ class LoginForm extends SpecialPage {
 		$this->mLoginattempt = $request->getCheck( 'wpLoginattempt' );
 		$this->mAction = $request->getVal( 'action' );
 		$this->mRemember = $request->getCheck( 'wpRemember' );
-		$this->mFromHTTP = $request->getBool( 'fromhttp', false );
+		$this->mFromHTTP = $request->getBool( 'fromhttp', false )
+			|| $request->getBool( 'wpFromhttp', false );
 		$this->mStickHTTPS = ( !$this->mFromHTTP && $request->getProtocol() === 'https' )
 			|| $request->getBool( 'wpForceHttps', false );
 		$this->mLanguage = $request->getText( 'uselang' );
@@ -128,6 +172,37 @@ class LoginForm extends SpecialPage {
 		$this->mReturnTo = $request->getVal( 'returnto', '' );
 		$this->mReturnToQuery = $request->getVal( 'returntoquery', '' );
 
+		// Show an error or warning passed on from a previous page
+		$entryError = $this->msg( $request->getVal( 'error', '' ) );
+		$entryWarning = $this->msg( $request->getVal( 'warning', '' ) );
+		// bc: provide login link as a parameter for messages where the translation
+		// was not updated
+		$loginreqlink = Linker::linkKnown(
+			$this->getPageTitle(),
+			$this->msg( 'loginreqlink' )->escaped(),
+			array(),
+			array(
+				'returnto' => $this->mReturnTo,
+				'returntoquery' => $this->mReturnToQuery,
+				'uselang' => $this->mLanguage,
+				'fromhttp' => $this->mFromHTTP ? '1' : '0',
+			)
+		);
+
+		// Only show valid error or warning messages.
+		if ( $entryError->exists()
+			&& in_array( $entryError->getKey(), self::getValidErrorMessages() )
+		) {
+			$this->mEntryErrorType = 'error';
+			$this->mEntryError = $entryError->rawParams( $loginreqlink )->escaped();
+
+		} elseif ( $entryWarning->exists()
+			&& in_array( $entryWarning->getKey(), self::getValidErrorMessages() )
+		) {
+			$this->mEntryErrorType = 'warning';
+			$this->mEntryError = $entryWarning->rawParams( $loginreqlink )->escaped();
+		}
+
 		if ( $wgEnableEmail ) {
 			$this->mEmail = $request->getText( 'wpEmail' );
 		} else {
@@ -165,8 +240,8 @@ class LoginForm extends SpecialPage {
 		}
 	}
 
-	/*
-	 * @param $subPage string|null
+	/**
+	 * @param string|null $subPage
 	 */
 	public function execute( $subPage ) {
 		if ( session_id() == '' ) {
@@ -182,6 +257,22 @@ class LoginForm extends SpecialPage {
 		}
 		$this->setHeaders();
 
+		// In the case where the user is already logged in, and was redirected to the login form from a
+		// page that requires login, do not show the login page. The use case scenario for this is when
+		// a user opens a large number of tabs, is redirected to the login page on all of them, and then
+		// logs in on one, expecting all the others to work properly.
+		//
+		// However, do show the form if it was visited intentionally (no 'returnto' is present). People
+		// who often switch between several accounts have grown accustomed to this behavior.
+		if (
+			$this->mType !== 'signup' &&
+			!$this->mPosted &&
+			$this->getUser()->isLoggedIn() &&
+			( $this->mReturnTo !== '' || $this->mReturnToQuery !== '' )
+		) {
+			$this->successfulLogin();
+		}
+
 		// If logging in and not on HTTPS, either redirect to it or offer a link.
 		global $wgSecureLogin;
 		if ( $this->mRequest->getProtocol() !== 'https' ) {
@@ -191,9 +282,13 @@ class LoginForm extends SpecialPage {
 				'returntoquery' => $this->mReturnToQuery !== '' ?
 					$this->mReturnToQuery : null,
 				'title' => null,
+				( $this->mEntryErrorType === 'error' ? 'error' : 'warning' ) => $this->mEntryError,
 			) + $this->mRequest->getQueryValues();
 			$url = $title->getFullURL( $query, false, PROTO_HTTPS );
-			if ( $wgSecureLogin && wfCanIPUseHTTPS( $this->getRequest()->getIP() ) ) {
+			if ( $wgSecureLogin
+				&& wfCanIPUseHTTPS( $this->getRequest()->getIP() )
+				&& !$this->mFromHTTP ) // Avoid infinite redirect
+			{
 				$url = wfAppendQuery( $url, 'fromhttp=1' );
 				$this->getOutput()->redirect( $url );
 				// Since we only do this redir to change proto, always vary
@@ -229,7 +324,7 @@ class LoginForm extends SpecialPage {
 				return;
 			}
 		}
-		$this->mainLoginForm( '' );
+		$this->mainLoginForm( $this->mEntryError, $this->mEntryErrorType );
 	}
 
 	/**
@@ -257,7 +352,7 @@ class LoginForm extends SpecialPage {
 		$u->saveSettings();
 		$result = $this->mailPasswordInternal( $u, false, 'createaccount-title', 'createaccount-text' );
 
-		wfRunHooks( 'AddNewAccount', array( $u, true ) );
+		Hooks::run( 'AddNewAccount', array( $u, true ) );
 		$u->addNewUserLogEntry( 'byemail', $this->mReason );
 
 		$out = $this->getOutput();
@@ -332,7 +427,7 @@ class LoginForm extends SpecialPage {
 			// which is needed or the personal links will be
 			// wrong.
 			$this->getContext()->setUser( $u );
-			wfRunHooks( 'AddNewAccount', array( $u, false ) );
+			Hooks::run( 'AddNewAccount', array( $u, false ) );
 			$u->addNewUserLogEntry( 'create' );
 			if ( $this->hasSessionCookie() ) {
 				$this->successfulCreation();
@@ -344,7 +439,7 @@ class LoginForm extends SpecialPage {
 			$out->setPageTitle( $this->msg( 'accountcreated' ) );
 			$out->addWikiMsg( 'accountcreatedtext', $u->getName() );
 			$out->addReturnTo( $this->getPageTitle() );
-			wfRunHooks( 'AddNewAccount', array( $u, false ) );
+			Hooks::run( 'AddNewAccount', array( $u, false ) );
 			$u->addNewUserLogEntry( 'create2', $this->mReason );
 		}
 
@@ -433,16 +528,8 @@ class LoginForm extends SpecialPage {
 			return Status::newFatal( 'sorbs_create_account_reason' );
 		}
 
-		// Normalize the name so that silly things don't cause "invalid username"
-		// errors. User::newFromName does some rather strict checking, rejecting
-		// e.g. leading/trailing/multiple spaces.
-		$title = Title::makeTitleSafe( NS_USER, $this->mUsername );
-		if ( !is_object( $title ) ) {
-			return Status::newFatal( 'noname' );
-		}
-
 		# Now create a dummy user ($u) and check if it is valid
-		$u = User::newFromName( $title->getText(), 'creatable' );
+		$u = User::newFromName( $this->mUsername, 'creatable' );
 		if ( !is_object( $u ) ) {
 			return Status::newFatal( 'noname' );
 		} elseif ( 0 != $u->idForName() ) {
@@ -458,14 +545,11 @@ class LoginForm extends SpecialPage {
 				return Status::newFatal( 'badretype' );
 			}
 
-			# check for minimal password length
-			$valid = $u->getPasswordValidity( $this->mPassword );
-			if ( $valid !== true ) {
-				if ( !is_array( $valid ) ) {
-					$valid = array( $valid, $wgMinimalPasswordLength );
-				}
-
-				return call_user_func_array( 'Status::newFatal', $valid );
+			# check for password validity, return a fatal Status if invalid
+			$validity = $u->checkPasswordValidity( $this->mPassword );
+			if ( !$validity->isGood() ) {
+				$validity->ok = false; // make sure this Status is fatal
+				return $validity;
 			}
 		}
 
@@ -486,7 +570,7 @@ class LoginForm extends SpecialPage {
 
 		$abortError = '';
 		$abortStatus = null;
-		if ( !wfRunHooks( 'AbortNewAccount', array( $u, &$abortError, &$abortStatus ) ) ) {
+		if ( !Hooks::run( 'AbortNewAccount', array( $u, &$abortError, &$abortStatus ) ) ) {
 			// Hook point to add extra creation throttles and blocks
 			wfDebug( "LoginForm::addNewAccountInternal: a hook blocked creation\n" );
 			if ( $abortStatus === null ) {
@@ -506,12 +590,12 @@ class LoginForm extends SpecialPage {
 		}
 
 		// Hook point to check for exempt from account creation throttle
-		if ( !wfRunHooks( 'ExemptFromAccountCreationThrottle', array( $ip ) ) ) {
+		if ( !Hooks::run( 'ExemptFromAccountCreationThrottle', array( $ip ) ) ) {
 			wfDebug( "LoginForm::exemptFromAccountCreationThrottle: a hook " .
 				"allowed account creation w/o throttle\n" );
 		} else {
 			if ( ( $wgAccountCreationThrottle && $currentUser->isPingLimitable() ) ) {
-				$key = wfMemcKey( 'acctcreate', 'ip', $ip );
+				$key = 'global:acctcreate:ip:' . $ip;
 				$value = $wgMemc->get( $key );
 				if ( !$value ) {
 					$wgMemc->set( $key, 0, 86400 );
@@ -536,9 +620,9 @@ class LoginForm extends SpecialPage {
 	 * Actually add a user to the database.
 	 * Give it a User object that has been initialised with a name.
 	 *
-	 * @param $u User object.
-	 * @param $autocreate boolean -- true if this is an autocreation via auth plugin
-	 * @return Status object, with the User object in the value member on success
+	 * @param User $u
+	 * @param bool $autocreate True if this is an autocreation via auth plugin
+	 * @return Status Status object, with the User object in the value member on success
 	 * @private
 	 */
 	function initUser( $u, $autocreate ) {
@@ -626,6 +710,14 @@ class LoginForm extends SpecialPage {
 		}
 
 		$u = User::newFromName( $this->mUsername );
+
+		// Give extensions a way to indicate the username has been updated,
+		// rather than telling the user the account doesn't exist.
+		if ( !Hooks::run( 'LoginUserMigrated', array( $u, &$msg ) ) ) {
+			$this->mAbortLoginErrorMsg = $msg;
+			return self::USER_MIGRATED;
+		}
+
 		if ( !( $u instanceof User ) || !User::isUsableName( $u->getName() ) ) {
 			return self::ILLEGAL;
 		}
@@ -645,7 +737,7 @@ class LoginForm extends SpecialPage {
 		// Give general extensions, such as a captcha, a chance to abort logins
 		$abort = self::ABORTED;
 		$msg = null;
-		if ( !wfRunHooks( 'AbortLogin', array( $u, $this->mPassword, &$abort, &$msg ) ) ) {
+		if ( !Hooks::run( 'AbortLogin', array( $u, $this->mPassword, &$abort, &$msg ) ) ) {
 			$this->mAbortLoginErrorMsg = $msg;
 
 			return $abort;
@@ -670,7 +762,7 @@ class LoginForm extends SpecialPage {
 				// As a side-effect, we can authenticate the user's e-mail ad-
 				// dress if it's not already done, since the temporary password
 				// was sent via e-mail.
-				if ( !$u->isEmailConfirmed() ) {
+				if ( !$u->isEmailConfirmed() && !wfReadOnly() ) {
 					$u->confirmEmail();
 					$u->saveSettings();
 				}
@@ -706,12 +798,12 @@ class LoginForm extends SpecialPage {
 
 			if ( $isAutoCreated ) {
 				// Must be run after $wgUser is set, for correct new user log
-				wfRunHooks( 'AuthPluginAutoCreate', array( $u ) );
+				Hooks::run( 'AuthPluginAutoCreate', array( $u ) );
 			}
 
 			$retval = self::SUCCESS;
 		}
-		wfRunHooks( 'LoginAuthenticateAudit', array( $u, $this->mPassword, $retval ) );
+		Hooks::run( 'LoginAuthenticateAudit', array( $u, $this->mPassword, $retval ) );
 
 		return $retval;
 	}
@@ -720,15 +812,16 @@ class LoginForm extends SpecialPage {
 	 * Increment the login attempt throttle hit count for the (username,current IP)
 	 * tuple unless the throttle was already reached.
 	 * @param string $username The user name
-	 * @return Bool|Integer The integer hit count or True if it is already at the limit
+	 * @return bool|int The integer hit count or True if it is already at the limit
 	 */
 	public static function incLoginThrottle( $username ) {
 		global $wgPasswordAttemptThrottle, $wgMemc, $wgRequest;
-		$username = trim( $username ); // sanity
+		$canUsername = User::getCanonicalName( $username, 'usable' );
+		$username = $canUsername !== false ? $canUsername : $username;
 
 		$throttleCount = 0;
 		if ( is_array( $wgPasswordAttemptThrottle ) ) {
-			$throttleKey = wfMemcKey( 'password-throttle', $wgRequest->getIP(), md5( $username ) );
+			$throttleKey = 'global:password-throttle:' . $wgRequest->getIP() . ':' . md5( $username );
 			$count = $wgPasswordAttemptThrottle['count'];
 			$period = $wgPasswordAttemptThrottle['seconds'];
 
@@ -752,9 +845,10 @@ class LoginForm extends SpecialPage {
 	 */
 	public static function clearLoginThrottle( $username ) {
 		global $wgMemc, $wgRequest;
-		$username = trim( $username ); // sanity
+		$canUsername = User::getCanonicalName( $username, 'usable' );
+		$username = $canUsername !== false ? $canUsername : $username;
 
-		$throttleKey = wfMemcKey( 'password-throttle', $wgRequest->getIP(), md5( $username ) );
+		$throttleKey = 'global:password-throttle:' . $wgRequest->getIP() . ':' . md5( $username );
 		$wgMemc->delete( $throttleKey );
 	}
 
@@ -762,9 +856,9 @@ class LoginForm extends SpecialPage {
 	 * Attempt to automatically create a user on login. Only succeeds if there
 	 * is an external authentication method which allows it.
 	 *
-	 * @param $user User
+	 * @param User $user
 	 *
-	 * @return integer Status code
+	 * @return int Status code
 	 */
 	function attemptAutoCreate( $user ) {
 		global $wgAuth;
@@ -792,7 +886,7 @@ class LoginForm extends SpecialPage {
 		}
 
 		$abortError = '';
-		if ( !wfRunHooks( 'AbortAutoAccount', array( $user, &$abortError ) ) ) {
+		if ( !Hooks::run( 'AbortAutoAccount', array( $user, &$abortError ) ) ) {
 			// Hook point to add extra creation throttles and blocks
 			wfDebug( "LoginForm::attemptAutoCreate: a hook blocked creation: $abortError\n" );
 			$this->mAbortLoginErrorMsg = $abortError;
@@ -821,7 +915,7 @@ class LoginForm extends SpecialPage {
 			case self::SUCCESS:
 				# We've verified now, update the real record
 				$user = $this->getUser();
-				$user->invalidateCache();
+				$user->touch();
 
 				if ( $user->requiresHTTPS() ) {
 					$this->mStickHTTPS = true;
@@ -922,7 +1016,17 @@ class LoginForm extends SpecialPage {
 				break;
 			case self::ABORTED:
 				$error = $this->mAbortLoginErrorMsg ?: 'login-abort-generic';
-				$this->mainLoginForm( $this->msg( $error )->text() );
+				$this->mainLoginForm( $this->msg( $error,
+						wfEscapeWikiText( $this->mUsername ) )->text() );
+				break;
+			case self::USER_MIGRATED:
+				$error = $this->mAbortLoginErrorMsg ?: 'login-migrated-generic';
+				$params = array();
+				if ( is_array( $error ) ) {
+					$error = array_shift( $this->mAbortLoginErrorMsg );
+					$params = $this->mAbortLoginErrorMsg;
+				}
+				$this->mainLoginForm( $this->msg( $error, $params )->text() );
 				break;
 			default:
 				throw new MWException( 'Unhandled case value' );
@@ -935,7 +1039,7 @@ class LoginForm extends SpecialPage {
 	 */
 	protected function resetLoginForm( Message $msg ) {
 		// Allow hooks to explain this password reset in more detail
-		wfRunHooks( 'LoginPasswordResetMessage', array( &$msg, $this->mUsername ) );
+		Hooks::run( 'LoginPasswordResetMessage', array( &$msg, $this->mUsername ) );
 		$reset = new SpecialChangePassword();
 		$derivative = new DerivativeContext( $this->getContext() );
 		$derivative->setTitle( $reset->getPageTitle() );
@@ -948,11 +1052,11 @@ class LoginForm extends SpecialPage {
 	}
 
 	/**
-	 * @param $u User object
-	 * @param $throttle Boolean
-	 * @param string $emailTitle message name of email title
-	 * @param string $emailText message name of email text
-	 * @return Status object
+	 * @param User $u
+	 * @param bool $throttle
+	 * @param string $emailTitle Message name of email title
+	 * @param string $emailText Message name of email text
+	 * @return Status
 	 */
 	function mailPasswordInternal( $u, $throttle = true, $emailTitle = 'passwordremindertitle',
 		$emailText = 'passwordremindertext'
@@ -968,7 +1072,7 @@ class LoginForm extends SpecialPage {
 		}
 
 		$currentUser = $this->getUser();
-		wfRunHooks( 'User::mailPasswordInternal', array( &$currentUser, &$ip, &$u ) );
+		Hooks::run( 'User::mailPasswordInternal', array( &$currentUser, &$ip, &$u ) );
 
 		$np = $u->randomPassword();
 		$u->setNewpassword( $np, $throttle );
@@ -999,10 +1103,10 @@ class LoginForm extends SpecialPage {
 		# Run any hooks; display injected HTML if any, else redirect
 		$currentUser = $this->getUser();
 		$injected_html = '';
-		wfRunHooks( 'UserLoginComplete', array( &$currentUser, &$injected_html ) );
+		Hooks::run( 'UserLoginComplete', array( &$currentUser, &$injected_html ) );
 
 		if ( $injected_html !== '' ) {
-			$this->displaySuccessfulAction( $this->msg( 'loginsuccesstitle' ),
+			$this->displaySuccessfulAction( 'success', $this->msg( 'loginsuccesstitle' ),
 				'loginsuccess', $injected_html );
 		} else {
 			$this->executeReturnTo( 'successredirect' );
@@ -1021,27 +1125,31 @@ class LoginForm extends SpecialPage {
 		$injected_html = '';
 		$welcome_creation_msg = 'welcomecreation-msg';
 
-		wfRunHooks( 'UserLoginComplete', array( &$currentUser, &$injected_html ) );
+		Hooks::run( 'UserLoginComplete', array( &$currentUser, &$injected_html ) );
 
 		/**
 		 * Let any extensions change what message is shown.
 		 * @see https://www.mediawiki.org/wiki/Manual:Hooks/BeforeWelcomeCreation
 		 * @since 1.18
 		 */
-		wfRunHooks( 'BeforeWelcomeCreation', array( &$welcome_creation_msg, &$injected_html ) );
+		Hooks::run( 'BeforeWelcomeCreation', array( &$welcome_creation_msg, &$injected_html ) );
 
-		$this->displaySuccessfulAction( $this->msg( 'welcomeuser', $this->getUser()->getName() ),
-			$welcome_creation_msg, $injected_html );
+		$this->displaySuccessfulAction(
+			'signup',
+			$this->msg( 'welcomeuser', $this->getUser()->getName() ),
+			$welcome_creation_msg, $injected_html
+		);
 	}
 
 	/**
-	 * Display an "successful action" page.
+	 * Display a "successful action" page.
 	 *
-	 * @param string|Message $title page's title
-	 * @param $msgname string
-	 * @param $injected_html string
+	 * @param string $type Condition of return to; see `executeReturnTo`
+	 * @param string|Message $title Page's title
+	 * @param string $msgname
+	 * @param string $injected_html
 	 */
-	private function displaySuccessfulAction( $title, $msgname, $injected_html ) {
+	private function displaySuccessfulAction( $type, $title, $msgname, $injected_html ) {
 		$out = $this->getOutput();
 		$out->setPageTitle( $title );
 		if ( $msgname ) {
@@ -1050,7 +1158,7 @@ class LoginForm extends SpecialPage {
 
 		$out->addHTML( $injected_html );
 
-		$this->executeReturnTo( 'success' );
+		$this->executeReturnTo( $type );
 	}
 
 	/**
@@ -1058,7 +1166,7 @@ class LoginForm extends SpecialPage {
 	 * there is a block on them or their IP which prevents account creation.  Note that
 	 * User::isBlockedFromCreateAccount(), which gets this block, ignores the 'hardblock'
 	 * setting on blocks (bug 13611).
-	 * @param $block Block the block causing this error
+	 * @param Block $block The block causing this error
 	 * @throws ErrorPageError
 	 */
 	function userBlockedMessage( Block $block ) {
@@ -1094,8 +1202,9 @@ class LoginForm extends SpecialPage {
 	 * Extensions can use this to reuse the "return to" logic after
 	 * inject steps (such as redirection) into the login process.
 	 *
-	 * @param $type string, one of the following:
+	 * @param string $type One of the following:
 	 *    - error: display a return to link ignoring $wgRedirectOnLogin
+	 *    - signup: display a return to link using $wgRedirectOnLogin if needed
 	 *    - success: display a return to link using $wgRedirectOnLogin if needed
 	 *    - successredirect: send an HTTP redirect using $wgRedirectOnLogin if needed
 	 * @param string $returnTo
@@ -1115,8 +1224,9 @@ class LoginForm extends SpecialPage {
 	/**
 	 * Add a "return to" link or redirect to it.
 	 *
-	 * @param $type string, one of the following:
+	 * @param string $type One of the following:
 	 *    - error: display a return to link ignoring $wgRedirectOnLogin
+	 *    - signup: display a return to link using $wgRedirectOnLogin if needed
 	 *    - success: display a return to link using $wgRedirectOnLogin if needed
 	 *    - successredirect: send an HTTP redirect using $wgRedirectOnLogin if needed
 	 */
@@ -1131,6 +1241,9 @@ class LoginForm extends SpecialPage {
 			$returnToQuery = wfCgiToArray( $this->mReturnToQuery );
 		}
 
+		// Allow modification of redirect behavior
+		Hooks::run( 'PostLoginRedirect', array( &$returnTo, &$returnToQuery, &$type ) );
+
 		$returnToTitle = Title::newFromText( $returnTo );
 		if ( !$returnToTitle ) {
 			$returnToTitle = Title::newMainPage();
@@ -1156,6 +1269,14 @@ class LoginForm extends SpecialPage {
 	}
 
 	/**
+	 * @param string $msg
+	 * @param string $msgtype
+	 * @throws ErrorPageError
+	 * @throws Exception
+	 * @throws FatalError
+	 * @throws MWException
+	 * @throws PermissionsError
+	 * @throws ReadOnlyError
 	 * @private
 	 */
 	function mainLoginForm( $msg, $msgtype = 'error' ) {
@@ -1197,6 +1318,8 @@ class LoginForm extends SpecialPage {
 		$out->addModuleStyles( array(
 			'mediawiki.ui',
 			'mediawiki.ui.button',
+			'mediawiki.ui.checkbox',
+			'mediawiki.ui.input',
 			'mediawiki.special.userlogin.common.styles'
 		) );
 		$out->addModules( array(
@@ -1217,7 +1340,7 @@ class LoginForm extends SpecialPage {
 				'mediawiki.special.userlogin.signup.styles'
 			) );
 
-			$template = new UsercreateTemplate();
+			$template = new UsercreateTemplate( $this->getConfig() );
 
 			// Must match number of benefits defined in messages
 			$template->set( 'benefitCount', 3 );
@@ -1230,7 +1353,7 @@ class LoginForm extends SpecialPage {
 				'mediawiki.special.userlogin.login.styles'
 			) );
 
-			$template = new UserloginTemplate();
+			$template = new UserloginTemplate( $this->getConfig() );
 
 			$q = 'action=submitlogin&type=login';
 			$linkq = 'type=signup';
@@ -1312,28 +1435,26 @@ class LoginForm extends SpecialPage {
 		}
 
 		$template->set( 'secureLoginUrl', $this->mSecureLoginUrl );
-		// Use loginend-https for HTTPS requests if it's not blank, loginend otherwise
-		// Ditto for signupend.  New forms use neither.
+		// Use signupend-https for HTTPS requests if it's not blank, signupend otherwise
 		$usingHTTPS = $this->mRequest->getProtocol() == 'https';
-		$loginendHTTPS = $this->msg( 'loginend-https' );
 		$signupendHTTPS = $this->msg( 'signupend-https' );
-		if ( $usingHTTPS && !$loginendHTTPS->isBlank() ) {
-			$template->set( 'loginend', $loginendHTTPS->parse() );
-		} else {
-			$template->set( 'loginend', $this->msg( 'loginend' )->parse() );
-		}
 		if ( $usingHTTPS && !$signupendHTTPS->isBlank() ) {
 			$template->set( 'signupend', $signupendHTTPS->parse() );
 		} else {
 			$template->set( 'signupend', $this->msg( 'signupend' )->parse() );
 		}
 
+		// If using HTTPS coming from HTTP, then the 'fromhttp' parameter must be preserved
+		if ( $usingHTTPS ) {
+			$template->set( 'fromhttp', $this->mFromHTTP );
+		}
+
 		// Give authentication and captcha plugins a chance to modify the form
 		$wgAuth->modifyUITemplate( $template, $this->mType );
 		if ( $this->mType == 'signup' ) {
-			wfRunHooks( 'UserCreateForm', array( &$template ) );
+			Hooks::run( 'UserCreateForm', array( &$template ) );
 		} else {
-			wfRunHooks( 'UserLoginForm', array( &$template ) );
+			Hooks::run( 'UserLoginForm', array( &$template ) );
 		}
 
 		$out->disallowUserJs(); // just in case...
@@ -1344,7 +1465,7 @@ class LoginForm extends SpecialPage {
 	 * Whether the login/create account form should display a link to the
 	 * other form (in addition to whatever the skin provides).
 	 *
-	 * @param $user User
+	 * @param User $user
 	 * @return bool
 	 */
 	private function showCreateOrLoginLink( &$user ) {
@@ -1375,7 +1496,7 @@ class LoginForm extends SpecialPage {
 
 	/**
 	 * Get the login token from the current session
-	 * @return Mixed
+	 * @return mixed
 	 */
 	public static function getLoginToken() {
 		global $wgRequest;
@@ -1403,7 +1524,7 @@ class LoginForm extends SpecialPage {
 
 	/**
 	 * Get the createaccount token from the current session
-	 * @return Mixed
+	 * @return mixed
 	 */
 	public static function getCreateaccountToken() {
 		global $wgRequest;
@@ -1435,11 +1556,13 @@ class LoginForm extends SpecialPage {
 		if ( $wgSecureLogin && !$this->mStickHTTPS ) {
 			$wgCookieSecure = false;
 		}
-
+		// Always make sure edit token is regenerated. (T114419)
+		$this->getRequest()->setSessionData( 'wsEditToken', null );
 		wfResetSessionID();
 	}
 
 	/**
+	 * @param string $type
 	 * @private
 	 */
 	function cookieRedirectCheck( $type ) {
@@ -1455,6 +1578,7 @@ class LoginForm extends SpecialPage {
 	}
 
 	/**
+	 * @param string $type
 	 * @private
 	 */
 	function onCookieRedirectCheck( $type ) {
