@@ -1,8 +1,7 @@
 <?php
 /** Belarusian (Taraškievica orthography) (беларуская (тарашкевіца)‎)
  *
- * See MessagesQqq.php for message documentation incl. usage of parameters
- * To improve a translation please visit http://translatewiki.net
+ * To improve a translation please visit https://translatewiki.net
  *
  * @ingroup Language
  * @file
@@ -73,7 +72,7 @@ $specialPageAliases = array(
 	'Mytalk'                    => array( 'Мае_размовы' ),
 	'Newimages'                 => array( 'Новыя_файлы' ),
 	'Newpages'                  => array( 'Новыя_старонкі' ),
-	'Popularpages'              => array( 'Папулярныя_старонкі' ),
+
 	'Protectedpages'            => array( 'Абароненыя_старонкі' ),
 	'Protectedtitles'           => array( 'Забароненыя_старонкі' ),
 	'Randompage'                => array( 'Выпадковая_старонка' ),
@@ -131,7 +130,6 @@ $magicWords = array(
 	'numberofusers'             => array( '1', 'КОЛЬКАСЬЦЬ_УДЗЕЛЬНІКАЎ', 'NUMBEROFUSERS' ),
 	'numberofactiveusers'       => array( '1', 'КОЛЬКАСЬЦЬ_АКТЫЎНЫХ_УДЗЕЛЬНІКАЎ', 'NUMBEROFACTIVEUSERS' ),
 	'numberofedits'             => array( '1', 'КОЛЬКАСЬЦЬ_РЭДАГАВАНЬНЯЎ', 'NUMBEROFEDITS' ),
-	'numberofviews'             => array( '1', 'КОЛЬКАСЬЦЬ_ПРАГЛЯДАЎ', 'NUMBEROFVIEWS' ),
 	'pagename'                  => array( '1', 'НАЗВА_СТАРОНКІ', 'PAGENAME' ),
 	'pagenamee'                 => array( '1', 'НАЗВА_СТАРОНКІ_2', 'PAGENAMEE' ),
 	'namespace'                 => array( '1', 'ПРАСТОРА_НАЗВАЎ', 'NAMESPACE' ),
