@@ -1,8 +1,7 @@
 <?php
 /** Japanese (日本語)
  *
- * See MessagesQqq.php for message documentation incl. usage of parameters
- * To improve a translation please visit http://translatewiki.net
+ * To improve a translation please visit https://translatewiki.net
  *
  * @ingroup Language
  * @file
@@ -61,7 +60,8 @@ $namespaceAliases = array(
 
 $specialPageAliases = array(
 	'Activeusers'               => array( '活動中の利用者', '活動中の利用者一覧' ),
-	'Allmessages'               => array( 'メッセージ一覧', 'システムメッセージの一覧', '表示メッセージの一覧' ),
+	'Allmessages'               => array( 'メッセージ一覧', 'システムメッセージの一覧', '表示メッセージの一覧', '全メッセージ' ),
+	'AllMyUploads'              => array( '自分の全アップロード' ),
 	'Allpages'                  => array( 'ページ一覧', '全ページ' ),
 	'Ancientpages'              => array( '更新されていないページ' ),
 	'Badtitle'                  => array( '不正なページ名' ),
@@ -78,6 +78,7 @@ $specialPageAliases = array(
 	'CreateAccount'             => array( 'アカウント作成', 'アカウントの作成' ),
 	'Deadendpages'              => array( '有効なページへのリンクがないページ', '行き止まりページ' ),
 	'DeletedContributions'      => array( '削除された投稿記録', '削除された投稿履歴', '削除歴' ),
+	'Diff'                      => array( '差分' ),
 	'DoubleRedirects'           => array( '二重リダイレクト', '二重転送' ),
 	'EditWatchlist'             => array( 'ウォッチリストの編集', 'ウォッチリスト編集' ),
 	'Emailuser'                 => array( 'メール送信', 'ウィキメール' ),
@@ -96,11 +97,13 @@ $specialPageAliases = array(
 	'Listfiles'                 => array( 'ファイル一覧', 'ファイルリスト' ),
 	'Listgrouprights'           => array( '利用者グループ権限', '利用者グループの権限一覧', '利用者権限一覧' ),
 	'Listredirects'             => array( 'リダイレクト一覧', 'リダイレクトの一覧', 'リダイレクトリスト' ),
+	'ListDuplicatedFiles'       => array( '重複ファイル一覧' ),
 	'Listusers'                 => array( '登録利用者一覧', '登録利用者の一覧' ),
 	'Lockdb'                    => array( 'データベースロック' ),
 	'Log'                       => array( 'ログ', '記録' ),
 	'Lonelypages'               => array( '孤立しているページ' ),
 	'Longpages'                 => array( '長いページ' ),
+	'MediaStatistics'           => array( 'メディア統計' ),
 	'MergeHistory'              => array( '履歴統合' ),
 	'MIMEsearch'                => array( 'MIME検索', 'MIMEタイプ検索' ),
 	'Mostcategories'            => array( 'カテゴリの多いページ', 'カテゴリの多い項目' ),
@@ -112,21 +115,23 @@ $specialPageAliases = array(
 	'Mostrevisions'             => array( '編集履歴の多いページ', '版の多い項目', '版の多いページ' ),
 	'Movepage'                  => array( '移動', 'ページの移動' ),
 	'Mycontributions'           => array( '自分の投稿記録' ),
+	'MyLanguage'                => array( '自分の言語' ),
 	'Mypage'                    => array( '利用者ページ', 'マイページ', 'マイ・ページ' ),
 	'Mytalk'                    => array( 'トークページ', '会話ページ', 'マイトーク', 'マイ・トーク' ),
 	'Myuploads'                 => array( '自分のアップロード記録' ),
 	'Newimages'                 => array( '新着ファイル', '新しいファイルの一覧', '新着画像展示室' ),
 	'Newpages'                  => array( '新しいページ' ),
 	'PagesWithProp'             => array( 'プロパティがあるページ' ),
+	'PageLanguage'              => array( 'ページ言語' ),
 	'PasswordReset'             => array( 'パスワード再設定', 'パスワードの再設定', 'パスワードのリセット', 'パスワードリセット' ),
 	'PermanentLink'             => array( '固定リンク', 'パーマリンク' ),
-	'Popularpages'              => array( '人気ページ' ),
+
 	'Preferences'               => array( '個人設定', 'オプション' ),
 	'Prefixindex'               => array( '前方一致ページ一覧', '始点指定ページ一覧' ),
 	'Protectedpages'            => array( '保護されているページ' ),
 	'Protectedtitles'           => array( '作成保護されているページ名' ),
 	'Randompage'                => array( 'おまかせ表示' ),
-	'RandomInCategory'          => array( 'カテゴリ内おまかせ表示', 'カテゴリ中のランダムなページ' ),
+	'RandomInCategory'          => array( 'カテゴリ内おまかせ表示' ),
 	'Randomredirect'            => array( 'おまかせリダイレクト', 'おまかせ転送' ),
 	'Recentchanges'             => array( '最近の更新', '最近更新したページ' ),
 	'Recentchangeslinked'       => array( '関連ページの更新状況', 'リンク先の更新状況' ),
@@ -138,6 +143,7 @@ $specialPageAliases = array(
 	'Specialpages'              => array( '特別ページ一覧' ),
 	'Statistics'                => array( '統計' ),
 	'Tags'                      => array( 'タグ一覧' ),
+	'TrackingCategories'        => array( '追跡カテゴリ' ),
 	'Unblock'                   => array( 'ブロック解除' ),
 	'Uncategorizedcategories'   => array( 'カテゴリ未導入のカテゴリ' ),
 	'Uncategorizedimages'       => array( 'カテゴリ未導入のファイル' ),
@@ -199,19 +205,21 @@ $magicWords = array(
 	'numberofusers'             => array( '1', '利用者数', 'NUMBEROFUSERS' ),
 	'numberofactiveusers'       => array( '1', '活動利用者数', '有効な利用者数', '有効利用者数', 'NUMBEROFACTIVEUSERS' ),
 	'numberofedits'             => array( '1', '編集回数', 'NUMBEROFEDITS' ),
-	'numberofviews'             => array( '1', '閲覧回数', 'NUMBEROFVIEWS' ),
 	'pagename'                  => array( '1', 'ページ名', 'PAGENAME' ),
 	'pagenamee'                 => array( '1', 'ページ名E', 'ページ名Ｅ', 'PAGENAMEE' ),
 	'namespace'                 => array( '1', '名前空間', 'NAMESPACE' ),
 	'namespacee'                => array( '1', '名前空間E', '名前空間Ｅ', 'NAMESPACEE' ),
+	'namespacenumber'           => array( '1', '名前空間番号', 'NAMESPACENUMBER' ),
 	'talkspace'                 => array( '1', 'トーク空間', 'ノート空間', '会話空間', 'トークスペース', 'TALKSPACE' ),
 	'talkspacee'                => array( '1', 'トーク空間E', 'トーク空間Ｅ', 'ノート空間E', '会話空間E', 'ノート空間Ｅ', '会話空間Ｅ', 'トークスペースE', 'トークスペースＥ', 'TALKSPACEE' ),
 	'subjectspace'              => array( '1', '主空間', '標準空間', '記事空間', 'SUBJECTSPACE', 'ARTICLESPACE' ),
 	'subjectspacee'             => array( '1', '主空間E', '標準空間E', '標準空間Ｅ', '記事空間E', '記事空間Ｅ', 'SUBJECTSPACEE', 'ARTICLESPACEE' ),
 	'fullpagename'              => array( '1', '完全なページ名', 'フルページ名', '完全な記事名', '完全記事名', 'FULLPAGENAME' ),
 	'fullpagenamee'             => array( '1', '完全なページ名E', 'フルページ名E', 'フルページ名Ｅ', '完全なページ名Ｅ', 'FULLPAGENAMEE' ),
-	'subpagename'               => array( '1', 'サブページ名', 'SUBPAGENAME' ),
-	'subpagenamee'              => array( '1', 'サブページ名E', 'サブページ名Ｅ', 'SUBPAGENAMEE' ),
+	'subpagename'               => array( '1', 'サブページ名', '下位ページ名', 'SUBPAGENAME' ),
+	'subpagenamee'              => array( '1', 'サブページ名E', 'サブページ名Ｅ', '下位ページ名E', '下位ページ名Ｅ', 'SUBPAGENAMEE' ),
+	'rootpagename'              => array( '1', 'ルートページ名', 'ROOTPAGENAME' ),
+	'rootpagenamee'             => array( '1', 'ルートページ名E', 'ルートページ名Ｅ', 'ROOTPAGENAMEE' ),
 	'basepagename'              => array( '1', '親ページ名', 'BASEPAGENAME' ),
 	'basepagenamee'             => array( '1', '親ページ名E', '親ページ名Ｅ', 'BASEPAGENAMEE' ),
 	'talkpagename'              => array( '1', 'トークページ名', '会話ページ名', 'TALKPAGENAME' ),
@@ -251,6 +259,7 @@ $magicWords = array(
 	'localurl'                  => array( '0', 'ローカルURL:', 'ローカルＵＲＬ：', 'LOCALURL:' ),
 	'localurle'                 => array( '0', 'ローカルURLE:', 'ローカルＵＲＬＥ：', 'LOCALURLE:' ),
 	'articlepath'               => array( '0', '記事パス', 'ARTICLEPATH' ),
+	'pageid'                    => array( '0', 'ページID', 'ページＩＤ', 'PAGEID' ),
 	'server'                    => array( '0', 'サーバー', 'サーバ', 'SERVER' ),
 	'servername'                => array( '0', 'サーバー名', 'サーバーネーム', 'サーバ名', 'サーバネーム', 'SERVERNAME' ),
 	'scriptpath'                => array( '0', 'スクリプトパス', 'SCRIPTPATH' ),
@@ -300,7 +309,7 @@ $magicWords = array(
 	'defaultsort'               => array( '1', 'デフォルトソート:', 'デフォルトソート：', 'デフォルトソートキー:', 'デフォルトカテゴリソート:', 'DEFAULTSORT:', 'DEFAULTSORTKEY:', 'DEFAULTCATEGORYSORT:' ),
 	'filepath'                  => array( '0', 'ファイルパス:', 'ファイルパス：', 'FILEPATH:' ),
 	'tag'                       => array( '0', 'タグ', 'tag' ),
-	'hiddencat'                 => array( '1', '__カテゴリ非表示__', '__カテ非表示__', '__非表示カテ__', '__HIDDENCAT__' ),
+	'hiddencat'                 => array( '1', '__カテゴリ非表示__', '__カテ非表示__', '__非表示カテ__', '__隠しカテゴリ__', '__HIDDENCAT__' ),
 	'pagesincategory'           => array( '1', 'カテゴリ内ページ数', 'PAGESINCATEGORY', 'PAGESINCAT' ),
 	'pagesize'                  => array( '1', 'ページサイズ', 'PAGESIZE' ),
 	'index'                     => array( '1', '__インデックス__', '＿＿インデックス＿＿', '__INDEX__' ),
