@@ -1,8 +1,7 @@
 <?php
 /** Swedish (svenska)
  *
- * See MessagesQqq.php for message documentation incl. usage of parameters
- * To improve a translation please visit http://translatewiki.net
+ * To improve a translation please visit https://translatewiki.net
  *
  * @ingroup Language
  * @file
@@ -91,24 +90,29 @@ $namespaceAliases = array(
 	'Hjälp_diskussion'     => NS_HELP_TALK
 );
 
-#!!# Translation <b>Prefixindex</b> for <a href="#mw-sp-magic-Prefixindex">Prefixindex</a> is not in normalised form, which is <b>PrefixIndex</b>
 $specialPageAliases = array(
+	'Activeusers'               => array( 'Aktiva_användare' ),
 	'Allmessages'               => array( 'Systemmeddelanden' ),
+	'AllMyUploads'              => array( 'Alla_mina_uppladdnignar', 'Alla_mina_filer' ),
 	'Allpages'                  => array( 'Alla_sidor' ),
 	'Ancientpages'              => array( 'Gamla_sidor' ),
+	'Badtitle'                  => array( 'Dålig_titel' ),
 	'Blankpage'                 => array( 'Tom_sida' ),
 	'Block'                     => array( 'Blockera' ),
 	'Booksources'               => array( 'Bokkällor' ),
 	'BrokenRedirects'           => array( 'Trasiga_omdirigeringar', 'Dåliga_omdirigeringar' ),
 	'Categories'                => array( 'Kategorier' ),
-	'ChangePassword'            => array( 'Återställ_lösenord' ),
+	'ChangeEmail'               => array( 'Ändra_e-postadress' ),
+	'ChangePassword'            => array( 'Ändra_lösenord' ),
+	'ComparePages'              => array( 'Jämför_sidor' ),
 	'Confirmemail'              => array( 'Bekräfta_epost' ),
 	'Contributions'             => array( 'Bidrag' ),
 	'CreateAccount'             => array( 'Skapa_konto' ),
 	'Deadendpages'              => array( 'Sidor_utan_länkar', 'Sidor_utan_länkar_från' ),
 	'DeletedContributions'      => array( 'Raderade_bidrag' ),
 	'DoubleRedirects'           => array( 'Dubbla_omdirigeringar' ),
-	'Emailuser'                 => array( 'E-mail' ),
+	'EditWatchlist'             => array( 'Redigera_bevakningslista' ),
+	'Emailuser'                 => array( 'E-post', 'E-mail' ),
 	'ExpandTemplates'           => array( 'Expandera_mallar', 'Utöka_mallar' ),
 	'Export'                    => array( 'Exportera' ),
 	'Fewestrevisions'           => array( 'Minst_versioner' ),
@@ -116,6 +120,7 @@ $specialPageAliases = array(
 	'Filepath'                  => array( 'Filsökväg' ),
 	'Import'                    => array( 'Importera' ),
 	'Invalidateemail'           => array( 'Ogiltigförklara_epost' ),
+	'JavaScriptTest'            => array( 'JavaScript_test' ),
 	'BlockList'                 => array( 'Blockeringslista' ),
 	'LinkSearch'                => array( 'Länksökning' ),
 	'Listadmins'                => array( 'Administratörer' ),
@@ -123,40 +128,52 @@ $specialPageAliases = array(
 	'Listfiles'                 => array( 'Fillista', 'Bildlista' ),
 	'Listgrouprights'           => array( 'Grupprättighetslista' ),
 	'Listredirects'             => array( 'Omdirigeringar' ),
+	'ListDuplicatedFiles'       => array( 'Lista_dubblettfiler' ),
 	'Listusers'                 => array( 'Användare', 'Användarlista' ),
 	'Lockdb'                    => array( 'Lås_databasen' ),
 	'Log'                       => array( 'Logg' ),
 	'Lonelypages'               => array( 'Föräldralösa_sidor', 'Övergivna_sidor', 'Sidor_utan_länkar_till' ),
 	'Longpages'                 => array( 'Långa_sidor' ),
+	'MediaStatistics'           => array( 'Media_statistik' ),
 	'MergeHistory'              => array( 'Slå_ihop_historik' ),
 	'MIMEsearch'                => array( 'MIME-sökning' ),
 	'Mostcategories'            => array( 'Flest_kategorier' ),
 	'Mostimages'                => array( 'Mest_länkade_filer', 'Flest_bilder' ),
+	'Mostinterwikis'            => array( 'Flest_interwikilänkar' ),
 	'Mostlinked'                => array( 'Mest_länkade_sidor' ),
 	'Mostlinkedcategories'      => array( 'Största_kategorier' ),
 	'Mostlinkedtemplates'       => array( 'Mest_använda_mallar' ),
 	'Mostrevisions'             => array( 'Flest_versioner' ),
 	'Movepage'                  => array( 'Flytta' ),
 	'Mycontributions'           => array( 'Mina_bidrag' ),
+	'MyLanguage'                => array( 'Mitt_språk' ),
 	'Mypage'                    => array( 'Min_sida' ),
 	'Mytalk'                    => array( 'Min_diskussion' ),
 	'Myuploads'                 => array( 'Mina_uppladdningar' ),
-	'Newimages'                 => array( 'Nya_bilder' ),
+	'Newimages'                 => array( 'Nya_filer', 'Nya_bilder' ),
 	'Newpages'                  => array( 'Nya_sidor' ),
-	'Popularpages'              => array( 'Populära_sidor' ),
+	'PagesWithProp'             => array( 'Sidor_med_en_sidegenskap' ),
+	'PageLanguage'              => array( 'Sidspråk' ),
+	'PasswordReset'             => array( 'Återställ_lösenord' ),
+	'PermanentLink'             => array( 'Permanent_länk' ),
+
 	'Preferences'               => array( 'Inställningar' ),
 	'Protectedpages'            => array( 'Skyddade_sidor' ),
 	'Protectedtitles'           => array( 'Skyddade_titlar' ),
 	'Randompage'                => array( 'Slumpsida' ),
+	'RandomInCategory'          => array( 'Slumpsida_i_kategori' ),
 	'Randomredirect'            => array( 'Slumpomdirigering' ),
 	'Recentchanges'             => array( 'Senaste_ändringar' ),
 	'Recentchangeslinked'       => array( 'Senaste_relaterade_ändringar' ),
+	'Redirect'                  => array( 'Omdirigering' ),
+	'ResetTokens'               => array( 'Återställ_nycklar' ),
 	'Revisiondelete'            => array( 'Radera_version' ),
 	'Search'                    => array( 'Sök' ),
 	'Shortpages'                => array( 'Korta_sidor' ),
 	'Specialpages'              => array( 'Specialsidor' ),
 	'Statistics'                => array( 'Statistik' ),
 	'Tags'                      => array( 'Märken', 'Taggar' ),
+	'TrackingCategories'        => array( 'Spårningskategorier' ),
 	'Unblock'                   => array( 'Avblockera' ),
 	'Uncategorizedcategories'   => array( 'Okategoriserade_kategorier' ),
 	'Uncategorizedimages'       => array( 'Okategoriserade_filer', 'Okategoriserade_bilder' ),
@@ -214,7 +231,6 @@ $magicWords = array(
 	'numberofusers'             => array( '1', 'ANTALANVÄNDARE', 'NUMBEROFUSERS' ),
 	'numberofactiveusers'       => array( '1', 'ANTALAKTIVAANVÄNDARE', 'NUMBEROFACTIVEUSERS' ),
 	'numberofedits'             => array( '1', 'ANTALREDIGERINGAR', 'NUMBEROFEDITS' ),
-	'numberofviews'             => array( '1', 'ANTALVISNINGAR', 'NUMBEROFVIEWS' ),
 	'pagename'                  => array( '1', 'SIDNAMN', 'PAGENAME' ),
 	'pagenamee'                 => array( '1', 'SIDNAMNE', 'PAGENAMEE' ),
 	'namespace'                 => array( '1', 'NAMNRYMD', 'NAMESPACE' ),
@@ -305,7 +321,7 @@ $magicWords = array(
 );
 
 $linkTrail = '/^([a-zåäöéÅÄÖÉ]+)(.*)$/sDu';
-$separatorTransformTable =  array(
+$separatorTransformTable = array(
 	',' => "\xc2\xa0", // @bug 2749
 	'.' => ','
 );
