@@ -1,8 +1,7 @@
 <?php
 /** Ukrainian (українська)
  *
- * See MessagesQqq.php for message documentation incl. usage of parameters
- * To improve a translation please visit http://translatewiki.net
+ * To improve a translation please visit https://translatewiki.net
  *
  * @ingroup Language
  * @file
@@ -117,6 +116,7 @@ $bookstoreList = array(
 $specialPageAliases = array(
 	'Activeusers'               => array( 'Активні_дописувачі' ),
 	'Allmessages'               => array( 'Системні_повідомлення' ),
+	'AllMyUploads'              => array( 'Усі_мої_файли' ),
 	'Allpages'                  => array( 'Усі_сторінки' ),
 	'Ancientpages'              => array( 'Давні_сторінки' ),
 	'Badtitle'                  => array( 'Помилковий_заголовок' ),
@@ -143,6 +143,7 @@ $specialPageAliases = array(
 	'Filepath'                  => array( 'Шлях_до_файлу' ),
 	'Import'                    => array( 'Імпорт' ),
 	'Invalidateemail'           => array( 'Неперевірена_email-адреса' ),
+	'JavaScriptTest'            => array( 'JavaScript_тест' ),
 	'BlockList'                 => array( 'Список_блокувань', 'Блокування', 'Блокування_IP-адрес' ),
 	'LinkSearch'                => array( 'Пошук_посилань' ),
 	'Listadmins'                => array( 'Список_адміністраторів' ),
@@ -150,6 +151,7 @@ $specialPageAliases = array(
 	'Listfiles'                 => array( 'Список_файлів' ),
 	'Listgrouprights'           => array( 'Список_прав_груп', 'Права_груп_користувачів' ),
 	'Listredirects'             => array( 'Список_перенаправлень' ),
+	'ListDuplicatedFiles'       => array( 'Список_дубльованих_файлів' ),
 	'Listusers'                 => array( 'Список_користувачів' ),
 	'Lockdb'                    => array( 'Заблокувати_базу_даних' ),
 	'Log'                       => array( 'Журнали' ),
@@ -166,6 +168,7 @@ $specialPageAliases = array(
 	'Mostrevisions'             => array( 'Найбільш_редаговані' ),
 	'Movepage'                  => array( 'Перейменувати' ),
 	'Mycontributions'           => array( 'Мій_внесок' ),
+	'MyLanguage'                => array( 'Моя_мова' ),
 	'Mypage'                    => array( 'Моя_сторінка' ),
 	'Mytalk'                    => array( 'Моє_обговорення' ),
 	'Myuploads'                 => array( 'Мої_завантаження' ),
@@ -173,7 +176,7 @@ $specialPageAliases = array(
 	'Newpages'                  => array( 'Нові_сторінки' ),
 	'PasswordReset'             => array( 'Скинути_пароль' ),
 	'PermanentLink'             => array( 'Постійне_посилання' ),
-	'Popularpages'              => array( 'Популярні_сторінки' ),
+
 	'Preferences'               => array( 'Налаштування' ),
 	'Prefixindex'               => array( 'Покажчик_за_початком_назви' ),
 	'Protectedpages'            => array( 'Захищені_сторінки' ),
@@ -182,6 +185,7 @@ $specialPageAliases = array(
 	'Randomredirect'            => array( 'Випадкове_перенаправлення' ),
 	'Recentchanges'             => array( 'Нові_редагування' ),
 	'Recentchangeslinked'       => array( 'Пов\'язані_редагування' ),
+	'Redirect'                  => array( 'Перенаправлення' ),
 	'Revisiondelete'            => array( 'Вилучити_редагування' ),
 	'Search'                    => array( 'Пошук' ),
 	'Shortpages'                => array( 'Короткі_сторінки' ),
@@ -237,19 +241,18 @@ $magicWords = array(
 	'localmonthname'            => array( '1', 'НАЗВА_ЛОКАЛЬНОГО_МІСЯЦЯ', 'НАЗВАНИЕ_МЕСТНОГО_МЕСЯЦА', 'LOCALMONTHNAME' ),
 	'localmonthnamegen'         => array( '1', 'НАЗВА_ЛОКАЛЬНОГО_МІСЯЦЯ_РОД', 'НАЗВАНИЕ_МЕСТНОГО_МЕСЯЦА_РОД', 'LOCALMONTHNAMEGEN' ),
 	'localmonthabbrev'          => array( '1', 'НАЗВА_ЛОКАЛЬНОГО_МІСЯЦЯ_АБР', 'НАЗВАНИЕ_МЕСТНОГО_МЕСЯЦА_АБР', 'LOCALMONTHABBREV' ),
-	'localday'                  => array( '1', 'ЛОКАЛЬНИЙ_ДЕНЬ', 'МЕСТНЫЙ_ДЕНЬ', 'LOCALDAY' ),
-	'localday2'                 => array( '1', 'ЛОКАЛЬНИЙ_ДЕНЬ_2', 'МЕСТНЫЙ_ДЕНЬ_2', 'LOCALDAY2' ),
-	'localdayname'              => array( '1', 'НАЗВА_ЛОКАЛЬНОГО_ДНЯ', 'НАЗВАНИЕ_МЕСТНОГО_ДНЯ', 'LOCALDAYNAME' ),
-	'localyear'                 => array( '1', 'ЛОКАЛЬНИЙ_РІК', 'МЕСТНЫЙ_ГОД', 'LOCALYEAR' ),
-	'localtime'                 => array( '1', 'ЛОКАЛЬНИЙ_ЧАС', 'МЕСТНОЕ_ВРЕМЯ', 'LOCALTIME' ),
-	'localhour'                 => array( '1', 'ЛОКАЛЬНА_ГОДИНА', 'МЕСТНЫЙ_ЧАС', 'LOCALHOUR' ),
+	'localday'                  => array( '1', 'ЛОКАЛЬНИЙ_ДЕНЬ', 'МІСЦЕВИЙ_ДЕНЬ', 'МЕСТНЫЙ_ДЕНЬ', 'LOCALDAY' ),
+	'localday2'                 => array( '1', 'ЛОКАЛЬНИЙ_ДЕНЬ_2', 'МІСЦЕВИЙ_ДЕНЬ_2', 'МЕСТНЫЙ_ДЕНЬ_2', 'LOCALDAY2' ),
+	'localdayname'              => array( '1', 'НАЗВА_ЛОКАЛЬНОГО_ДНЯ', 'НАЗВА_МІСЦЕВОГО_ДНЯ', 'НАЗВАНИЕ_МЕСТНОГО_ДНЯ', 'LOCALDAYNAME' ),
+	'localyear'                 => array( '1', 'ЛОКАЛЬНИЙ_РІК', 'МІСЦЕВИЙ_РІК', 'МЕСТНЫЙ_ГОД', 'LOCALYEAR' ),
+	'localtime'                 => array( '1', 'ЛОКАЛЬНИЙ_ЧАС', 'МІСЦЕВИЙ_ЧАС', 'МЕСТНОЕ_ВРЕМЯ', 'LOCALTIME' ),
+	'localhour'                 => array( '1', 'ЛОКАЛЬНА_ГОДИНА', 'МІСЦЕВА_ГОДИНА', 'МЕСТНЫЙ_ЧАС', 'LOCALHOUR' ),
 	'numberofpages'             => array( '1', 'КІЛЬКІСТЬ_СТОРІНОК', 'КОЛИЧЕСТВО_СТРАНИЦ', 'NUMBEROFPAGES' ),
 	'numberofarticles'          => array( '1', 'КІЛЬКІСТЬ_СТАТЕЙ', 'КОЛИЧЕСТВО_СТАТЕЙ', 'NUMBEROFARTICLES' ),
 	'numberoffiles'             => array( '1', 'КІЛЬКІСТЬ_ФАЙЛІВ', 'КОЛИЧЕСТВО_ФАЙЛОВ', 'NUMBEROFFILES' ),
 	'numberofusers'             => array( '1', 'КІЛЬКІСТЬ_КОРИСТУВАЧІВ', 'КОЛИЧЕСТВО_УЧАСТНИКОВ', 'NUMBEROFUSERS' ),
 	'numberofactiveusers'       => array( '1', 'КІЛЬКІСТЬ_АКТИВНИХ_КОРИСТУВАЧІВ', 'КОЛИЧЕСТВО_АКТИВНЫХ_УЧАСТНИКОВ', 'NUMBEROFACTIVEUSERS' ),
 	'numberofedits'             => array( '1', 'КІЛЬКІСТЬ_РЕДАГУВАНЬ', 'КОЛИЧЕСТВО_ПРАВОК', 'NUMBEROFEDITS' ),
-	'numberofviews'             => array( '1', 'КІЛЬКІСТЬ_ПЕРЕГЛЯДІВ', 'КОЛИЧЕСТВО_ПРОСМОТРОВ', 'NUMBEROFVIEWS' ),
 	'pagename'                  => array( '1', 'НАЗВА_СТОРІНКИ', 'НАЗВАНИЕ_СТРАНИЦЫ', 'PAGENAME' ),
 	'pagenamee'                 => array( '1', 'НАЗВА_СТОРІНКИ_2', 'НАЗВАНИЕ_СТРАНИЦЫ_2', 'PAGENAMEE' ),
 	'namespace'                 => array( '1', 'ПРОСТІР_НАЗВ', 'ПРОСТРАНСТВО_ИМЁН', 'NAMESPACE' ),
@@ -266,8 +269,8 @@ $magicWords = array(
 	'basepagenamee'             => array( '1', 'ОСНОВА_НАЗВИ_ПІДСТОРІНКИ_2', 'ОСНОВА_НАЗВАНИЯ_СТРАНИЦЫ_2', 'BASEPAGENAMEE' ),
 	'talkpagename'              => array( '1', 'НАЗВА_СТОРІНКИ_ОБГОВОРЕННЯ', 'НАЗВАНИЕ_СТРАНИЦЫ_ОБСУЖДЕНИЯ', 'TALKPAGENAME' ),
 	'talkpagenamee'             => array( '1', 'НАЗВА_СТОРІНКИ_ОБГОВОРЕННЯ_2', 'НАЗВАНИЕ_СТРАНИЦЫ_ОБСУЖДЕНИЯ_2', 'TALKPAGENAMEE' ),
-	'subjectpagename'           => array( '1', 'НАЗВА_СТАТТІ', 'НАЗВАНИЕ_СТРАНИЦЫ_СТАТЬИ', 'SUBJECTPAGENAME', 'ARTICLEPAGENAME' ),
-	'subjectpagenamee'          => array( '1', 'НАЗВА_СТАТТІ_2', 'НАЗВАНИЕ_СТРАНИЦЫ_СТАТЬИ_2', 'SUBJECTPAGENAMEE', 'ARTICLEPAGENAMEE' ),
+	'subjectpagename'           => array( '1', 'НАЗВА_СТАТТІ', 'НАЗВА_СТОРІНКИ_СТАТТІ', 'НАЗВАНИЕ_СТРАНИЦЫ_СТАТЬИ', 'SUBJECTPAGENAME', 'ARTICLEPAGENAME' ),
+	'subjectpagenamee'          => array( '1', 'НАЗВА_СТАТТІ_2', 'НАЗВА_СТОРІНКИ_СТАТТІ_2', 'НАЗВАНИЕ_СТРАНИЦЫ_СТАТЬИ_2', 'SUBJECTPAGENAMEE', 'ARTICLEPAGENAMEE' ),
 	'msg'                       => array( '0', 'ПОВІД:', 'ПОВІДОМЛЕННЯ:', 'СООБЩЕНИЕ:', 'СООБЩ:', 'MSG:' ),
 	'subst'                     => array( '0', 'ПІДСТ:', 'ПІДСТАНОВКА:', 'ПОДСТАНОВКА:', 'ПОДСТ:', 'SUBST:' ),
 	'safesubst'                 => array( '0', 'БЕЗПЕЧНА_ПІДСТАНОВКА:', 'ЗАЩПОДСТ:', 'SAFESUBST:' ),
@@ -281,9 +284,9 @@ $magicWords = array(
 	'img_center'                => array( '1', 'центр', 'center', 'centre' ),
 	'img_framed'                => array( '1', 'обрамити', 'рамка', 'обрамить', 'framed', 'enframed', 'frame' ),
 	'img_frameless'             => array( '1', 'безрамки', 'frameless' ),
-	'img_page'                  => array( '1', 'сторінка=$1', 'сторінка_$1', 'страница=$1', 'страница_$1', 'страница $1', 'page=$1', 'page $1' ),
-	'img_upright'               => array( '1', 'зверхуправоруч', 'зверхуправоруч=$1', 'зверхуправоруч_$1', 'сверхусправа', 'сверхусправа=$1', 'сверхусправа_$1', 'сверхусправа $1', 'upright', 'upright=$1', 'upright $1' ),
-	'img_border'                => array( '1', 'межа', 'граница', 'border' ),
+	'img_page'                  => array( '1', 'сторінка=$1', 'сторінка_$1', 'страница=$1', 'страница $1', 'page=$1', 'page $1' ),
+	'img_upright'               => array( '1', 'зверхуправоруч', 'зверхуправоруч=$1', 'зверхуправоруч_$1', 'сверхусправа', 'сверхусправа=$1', 'сверхусправа $1', 'upright', 'upright=$1', 'upright $1' ),
+	'img_border'                => array( '1', 'межа', 'границя', 'граница', 'border' ),
 	'img_baseline'              => array( '1', 'основа', 'основание', 'baseline' ),
 	'img_sub'                   => array( '1', 'під', 'под', 'sub' ),
 	'img_super'                 => array( '1', 'над', 'super', 'sup' ),
@@ -300,6 +303,8 @@ $magicWords = array(
 	'nse'                       => array( '0', 'ПН_2:', 'ПИК:', 'NSE:' ),
 	'localurl'                  => array( '0', 'ЛОКАЛЬНА_АДРЕСА:', 'ЛОКАЛЬНЫЙ_АДРЕС:', 'LOCALURL:' ),
 	'localurle'                 => array( '0', 'ЛОКАЛЬНА_АДРЕСА_2:', 'ЛОКАЛЬНЫЙ_АДРЕС_2:', 'LOCALURLE:' ),
+	'articlepath'               => array( '0', 'ШЛЯХ_ДО_СТАТТІ', 'ПУТЬ_К_СТАТЬЕ', 'ARTICLEPATH' ),
+	'pageid'                    => array( '0', 'ІДЕНТИФІКАТОР_СТОРІНКИ', 'ИДЕНТИФИКАТОР_СТРАНИЦЫ', 'PAGEID' ),
 	'server'                    => array( '0', 'СЕРВЕР', 'SERVER' ),
 	'servername'                => array( '0', 'НАЗВА_СЕРВЕРА', 'НАЗВАНИЕ_СЕРВЕРА', 'SERVERNAME' ),
 	'scriptpath'                => array( '0', 'ШЛЯХ_ДО_СКРИПТУ', 'ПУТЬ_К_СКРИПТУ', 'SCRIPTPATH' ),
@@ -323,10 +328,10 @@ $magicWords = array(
 	'plural'                    => array( '0', 'МНОЖИНА:', 'МНОЖЕСТВЕННОЕ_ЧИСЛО:', 'PLURAL:' ),
 	'fullurl'                   => array( '0', 'ПОВНА_АДРЕСА:', 'ПОЛНЫЙ_АДРЕС:', 'FULLURL:' ),
 	'fullurle'                  => array( '0', 'ПОВНА_АДРЕСА_2:', 'ПОЛНЫЙ_АДРЕС_2:', 'FULLURLE:' ),
-	'lcfirst'                   => array( '0', 'НР_ПЕРША:', 'ПЕРША_БУКВА_МАЛА:', 'ПЕРВАЯ_БУКВА_МАЛЕНЬКАЯ:', 'LCFIRST:' ),
-	'ucfirst'                   => array( '0', 'ВР_ПЕРША:', 'ПЕРША_БУКВА_ВЕЛИКА:', 'ПЕРВАЯ_БУКВА_БОЛЬШАЯ:', 'UCFIRST:' ),
-	'lc'                        => array( '0', 'НР:', 'НИЖНІЙ_РЕГІСТР:', 'МАЛИМИ_БУКВАМИ:', 'МАЛЕНЬКИМИ_БУКВАМИ:', 'LC:' ),
-	'uc'                        => array( '0', 'ВР:', 'ВЕРХНІЙ_РЕГІСТР:', 'ВЕЛИКИМИ_БУКВАМИ:', 'БОЛЬШИМИ_БУКВАМИ:', 'UC:' ),
+	'lcfirst'                   => array( '0', 'НР_ПЕРША:', 'ПЕРША_БУКВА_МАЛА:', 'ПЕРША_ЛІТЕРА_МАЛА:', 'МАЛА_ПЕРША_ЛІТЕРА:', 'ПЕРВАЯ_БУКВА_МАЛЕНЬКАЯ:', 'LCFIRST:' ),
+	'ucfirst'                   => array( '0', 'ВР_ПЕРША:', 'ПЕРША_БУКВА_ВЕЛИКА:', 'ПЕША_ДІТЕРА_ВЕЛИКА:', 'ВЕЛИКА_ПЕРША_ЛІТЕРА:', 'ПЕРВАЯ_БУКВА_БОЛЬШАЯ:', 'UCFIRST:' ),
+	'lc'                        => array( '0', 'НР:', 'НИЖНІЙ_РЕГІСТР:', 'МАЛИМИ_БУКВАМИ:', 'МАЛИМИ_ЛІТЕРАМИ:', 'МАЛЕНЬКИМИ_БУКВАМИ:', 'LC:' ),
+	'uc'                        => array( '0', 'ВР:', 'ВЕРХНІЙ_РЕГІСТР:', 'ВЕЛИКИМИ_БУКВАМИ:', 'ВЕЛИКИМИ_ЛІТЕРАМИ:', 'БОЛЬШИМИ_БУКВАМИ:', 'UC:' ),
 	'raw'                       => array( '0', 'НЕОБРОБ:', 'НЕОБРАБ:', 'RAW:' ),
 	'displaytitle'              => array( '1', 'ПОКАЗАТИ_ЗАГОЛОВОК', 'ПОКАЗАТЬ_ЗАГОЛОВОК', 'DISPLAYTITLE' ),
 	'rawsuffix'                 => array( '1', 'Н', 'R' ),
@@ -346,11 +351,11 @@ $magicWords = array(
 	'padleft'                   => array( '0', 'ЗАПОВНИТИ_ЛІВОРУЧ', 'ЗАПОЛНИТЬ_СЛЕВА', 'PADLEFT' ),
 	'padright'                  => array( '0', 'ЗАПОВНИТИ_ПРАВОРУЧ', 'ЗАПОЛНИТЬ_СПРАВА', 'PADRIGHT' ),
 	'special'                   => array( '0', 'спеціальна', 'служебная', 'special' ),
-	'defaultsort'               => array( '1', 'СТАНДАРТНЕ_СОРТУВАННЯ:', 'СОРТУВАННЯ:', 'СОРТИРОВКА_ПО_УМОЛЧАНИЮ', 'КЛЮЧ_СОРТИРОВКИ', 'DEFAULTSORT:', 'DEFAULTSORTKEY:', 'DEFAULTCATEGORYSORT:' ),
+	'defaultsort'               => array( '1', 'СТАНДАРТНЕ_СОРТУВАННЯ:_КЛЮЧ_СОРТУВАННЯ', 'СОРТИРОВКА_ПО_УМОЛЧАНИЮ', 'КЛЮЧ_СОРТИРОВКИ', 'DEFAULTSORT:', 'DEFAULTSORTKEY:', 'DEFAULTCATEGORYSORT:' ),
 	'filepath'                  => array( '0', 'ШЛЯХ_ДО_ФАЙЛУ:', 'ПУТЬ_К_ФАЙЛУ:', 'FILEPATH:' ),
-	'tag'                       => array( '0', 'тег', 'мітка', 'метка', 'тэг', 'tag' ),
-	'hiddencat'                 => array( '1', '__ПРИХОВ_КАТ__', '__СКРЫТАЯ_КАТЕГОРИЯ__', '__HIDDENCAT__' ),
-	'pagesincategory'           => array( '1', 'СТОР_В_КАТ', 'СТОР_У_КАТ', 'СТРАНИЦ_В_КАТЕГОРИИ', 'PAGESINCATEGORY', 'PAGESINCAT' ),
+	'tag'                       => array( '0', 'тег', 'мітка', 'теґ', 'метка', 'тэг', 'tag' ),
+	'hiddencat'                 => array( '1', '__ПРИХОВ_КАТ__', '__ПРИХОВАНА_КАТЕГОРІЯ__', '__СКРЫТАЯ_КАТЕГОРИЯ__', '__HIDDENCAT__' ),
+	'pagesincategory'           => array( '1', 'СТОР_В_КАТ', 'СТОР_У_КАТ', 'СТОРІНОК_У_КАТЕГОРІЇ', 'СТРАНИЦ_В_КАТЕГОРИИ', 'PAGESINCATEGORY', 'PAGESINCAT' ),
 	'pagesize'                  => array( '1', 'РОЗМІР', 'РОЗМІР_СТОРІНКИ', 'РАЗМЕР_СТРАНИЦЫ', 'PAGESIZE' ),
 	'index'                     => array( '1', '__ІНДЕКС__', '__ИНДЕКС__', '__INDEX__' ),
 	'noindex'                   => array( '1', '__БЕЗ_ІНДЕКСУ__', '__БЕЗ_ИНДЕКСА__', '__NOINDEX__' ),
@@ -361,6 +366,9 @@ $magicWords = array(
 	'url_path'                  => array( '0', 'ШЛЯХ', 'ПУТЬ', 'PATH' ),
 	'url_wiki'                  => array( '0', 'ВІКІ', 'ВИКИ', 'WIKI' ),
 	'url_query'                 => array( '0', 'ЗАПИТ', 'ЗАПРОС', 'QUERY' ),
+	'pagesincategory_all'       => array( '0', 'усе', 'все', 'all' ),
+	'pagesincategory_pages'     => array( '0', 'сторінки', 'страницы', 'pages' ),
+	'pagesincategory_subcats'   => array( '0', 'підкатегорії', 'подкатегории', 'subcats' ),
 );
 
 $linkTrail = '/^([a-zабвгґдеєжзиіїйклмнопрстуфхцчшщьєюяёъы“»]+)(.*)$/sDu';
