@@ -42,7 +42,8 @@ class RebuildAll extends Maintenance {
 	public function execute() {
 		// Rebuild the text index
 		if ( wfGetDB( DB_SLAVE )->getType() != 'postgres' ) {
-			$this->output( "** Rebuilding fulltext search index (if you abort this will break searching; run this script again to fix):\n" );
+			$this->output( "** Rebuilding fulltext search index (if you abort "
+				. "this will break searching; run this script again to fix):\n" );
 			$rebuildText = $this->runChild( 'RebuildTextIndex', 'rebuildtextindex.php' );
 			$rebuildText->execute();
 		}
@@ -53,7 +54,8 @@ class RebuildAll extends Maintenance {
 		$rebuildRC->execute();
 
 		// Rebuild link tables
-		$this->output( "\n\n** Rebuilding links tables -- this can take a long time. It should be safe to abort via ctrl+C if you get bored.\n" );
+		$this->output( "\n\n** Rebuilding links tables -- this can take a long time. "
+			. "It should be safe to abort via ctrl+C if you get bored.\n" );
 		$rebuildLinks = $this->runChild( 'RefreshLinks', 'refreshLinks.php' );
 		$rebuildLinks->execute();
 
