@@ -15,7 +15,7 @@
  */
 /**
  *
- * @description Create a sortable table with multi-column sorting capabilitys
+ * @description Create a sortable table with multi-column sorting capabilities
  *
  * @example $( 'table' ).tablesorter();
  * @desc Create a simple tablesorter interface.
@@ -35,15 +35,9 @@
  *         to sortable tr elements in the thead on a descending sort. Default
  *         value: "headerSortDown"
  *
- * @option String sortInitialOrder ( optional ) A string of the inital sorting
- *         order can be asc or desc. Default value: "asc"
- *
  * @option String sortMultisortKey ( optional ) A string of the multi-column sort
  *         key. Default value: "shiftKey"
  *
- * @option Boolean sortLocaleCompare ( optional ) Boolean flag indicating whatever
- *         to use String.localeCampare method or not. Set to false.
- *
  * @option Boolean cancelSelection ( optional ) Boolean flag indicating if
  *         tablesorter should cancel selection of the table headers text.
  *         Default value: true
@@ -53,9 +47,6 @@
  *         { <Integer column index>: <String 'asc' or 'desc'> }
  *         Default value: []
  *
- * @option Boolean debug ( optional ) Boolean flag indicating if tablesorter
- *         should display debuging information usefull for development.
- *
  * @event sortEnd.tablesorter: Triggered as soon as any sorting has been applied.
  *
  * @type jQuery
@@ -103,7 +94,7 @@
 			} else if ( node.tagName.toLowerCase() === 'img' ) {
 				return $node.attr( 'alt' ) || ''; // handle undefined alt
 			} else {
-				return $.map( $.makeArray( node.childNodes ), function( elem ) {
+				return $.map( $.makeArray( node.childNodes ), function ( elem ) {
 					// 1 is for document.ELEMENT_NODE (the constant is undefined on old browsers)
 					if ( elem.nodeType === 1 ) {
 						return getElementSortKey( elem );
@@ -131,7 +122,7 @@
 				nodeValue = '';
 			}
 
-			if ( nodeValue !== '') {
+			if ( nodeValue !== '' ) {
 				if ( parsers[i].is( nodeValue, table ) ) {
 					concurrent++;
 					rowIndex++;
@@ -160,15 +151,14 @@
 	}
 
 	function buildParserCache( table, $headers ) {
-		var rows = table.tBodies[0].rows,
-			sortType,
+		var sortType, cells, len, i, parser,
+			rows = table.tBodies[0].rows,
 			parsers = [];
 
 		if ( rows[0] ) {
 
-			var cells = rows[0].cells,
-				len = cells.length,
-				i, parser;
+			cells = rows[0].cells;
+			len = cells.length;
 
 			for ( i = 0; i < len; i++ ) {
 				parser = false;
@@ -190,23 +180,25 @@
 	/* Other utility functions */
 
 	function buildCache( table ) {
-		var totalRows = ( table.tBodies[0] && table.tBodies[0].rows.length ) || 0,
+		var i, j, $row, cols,
+			totalRows = ( table.tBodies[0] && table.tBodies[0].rows.length ) || 0,
 			totalCells = ( table.tBodies[0].rows[0] && table.tBodies[0].rows[0].cells.length ) || 0,
-			parsers = table.config.parsers,
+			config = $( table ).data( 'tablesorter' ).config,
+			parsers = config.parsers,
 			cache = {
 				row: [],
 				normalized: []
 			};
 
-		for ( var i = 0; i < totalRows; ++i ) {
+		for ( i = 0; i < totalRows; ++i ) {
 
 			// Add the table data to main data array
-			var $row = $( table.tBodies[0].rows[i] ),
-				cols = [];
+			$row = $( table.tBodies[0].rows[i] );
+			cols = [];
 
 			// if this is a child row, add it to the last row's children and
 			// continue to the next row
-			if ( $row.hasClass( table.config.cssChildRow ) ) {
+			if ( $row.hasClass( config.cssChildRow ) ) {
 				cache.row[cache.row.length - 1] = cache.row[cache.row.length - 1].add( $row );
 				// go to the next for loop
 				continue;
@@ -214,7 +206,7 @@
 
 			cache.row.push( $row );
 
-			for ( var j = 0; j < totalCells; ++j ) {
+			for ( j = 0; j < totalCells; ++j ) {
 				cols.push( parsers[j].format( getElementSortKey( $row[0].cells[j] ), table, $row[0].cells[j] ) );
 			}
 
@@ -262,52 +254,56 @@
 	function emulateTHeadAndFoot( $table ) {
 		var $thead, $tfoot, i, len,
 			$rows = $table.find( '> tbody > tr' );
-		if ( !$table.get(0).tHead ) {
+		if ( !$table.get( 0 ).tHead ) {
 			$thead = $( '<thead>' );
 			$rows.each( function () {
-				if ( $(this).children( 'td' ).length ) {
+				if ( $( this ).children( 'td' ).length ) {
 					// This row contains a <td>, so it's not a header row
 					// Stop here
 					return false;
 				}
 				$thead.append( this );
 			} );
-			$table.find(' > tbody:first').before( $thead );
+			$table.find( ' > tbody:first' ).before( $thead );
 		}
-		if ( !$table.get(0).tFoot ) {
+		if ( !$table.get( 0 ).tFoot ) {
 			$tfoot = $( '<tfoot>' );
 			len = $rows.length;
 			for ( i = len - 1; i >= 0; i-- ) {
-				if ( $( $rows[i] ).children( 'td' ).length ){
+				if ( $( $rows[i] ).children( 'td' ).length ) {
 					break;
 				}
-				$tfoot.prepend( $( $rows[i] ));
+				$tfoot.prepend( $( $rows[i] ) );
 			}
 			$table.append( $tfoot );
 		}
 	}
 
 	function buildHeaders( table, msg ) {
-		var maxSeen = 0,
+		var config = $( table ).data( 'tablesorter' ).config,
+			maxSeen = 0,
 			colspanOffset = 0,
 			columns,
 			i,
+			$cell,
+			rowspan,
+			colspan,
+			headerCount,
+			longestTR,
+			exploded,
 			$tableHeaders = $( [] ),
 			$tableRows = $( 'thead:eq(0) > tr', table );
 		if ( $tableRows.length <= 1 ) {
 			$tableHeaders = $tableRows.children( 'th' );
 		} else {
-			var rowspan,
-				colspan,
-				headerCount,
-				longestTR,
-				matrixRowIndex,
-				matrixColumnIndex,
-				exploded = [];
+			exploded = [];
 
 			// Loop through all the dom cells of the thead
 			$tableRows.each( function ( rowIndex, row ) {
-				$.each( row.cells, function( columnIndex, cell ) {
+				$.each( row.cells, function ( columnIndex, cell ) {
+					var matrixRowIndex,
+						matrixColumnIndex;
+
 					rowspan = Number( cell.rowSpan );
 					colspan = Number( cell.colSpan );
 
@@ -330,42 +326,43 @@
 			} );
 			// We want to find the row that has the most columns (ignoring colspan)
 			$.each( exploded, function ( index, cellArray ) {
-				headerCount = $.unique( $(cellArray) ).length;
+				headerCount = $( uniqueElements( cellArray ) ).filter( 'th' ).length;
 				if ( headerCount >= maxSeen ) {
 					maxSeen = headerCount;
 					longestTR = index;
 				}
 			} );
 			// We cannot use $.unique() here because it sorts into dom order, which is undesirable
-			$tableHeaders = $( uniqueElements( exploded[longestTR] ) );
+			$tableHeaders = $( uniqueElements( exploded[longestTR] ) ).filter( 'th' );
 		}
 
 		// as each header can span over multiple columns (using colspan=N),
 		// we have to bidirectionally map headers to their columns and columns to their headers
-		table.headerToColumns = [];
-		table.columnToHeader = [];
-
 		$tableHeaders.each( function ( headerIndex ) {
+			$cell = $( this );
 			columns = [];
+
 			for ( i = 0; i < this.colSpan; i++ ) {
-				table.columnToHeader[ colspanOffset + i ] = headerIndex;
+				config.columnToHeader[ colspanOffset + i ] = headerIndex;
 				columns.push( colspanOffset + i );
 			}
 
-			table.headerToColumns[ headerIndex ] = columns;
+			config.headerToColumns[ headerIndex ] = columns;
 			colspanOffset += this.colSpan;
 
-			this.headerIndex = headerIndex;
-			this.order = 0;
-			this.count = 0;
+			$cell.data( {
+				headerIndex: headerIndex,
+				order: 0,
+				count: 0
+			} );
 
-			if ( $( this ).hasClass( table.config.unsortableClass ) ) {
-				this.sortDisabled = true;
+			if ( $cell.hasClass( config.unsortableClass ) ) {
+				$cell.data( 'sortDisabled', true );
 			}
 
-			if ( !this.sortDisabled ) {
-				$( this )
-					.addClass( table.config.cssHeader )
+			if ( !$cell.data( 'sortDisabled' ) ) {
+				$cell
+					.addClass( config.cssHeader )
 					.prop( 'tabIndex', 0 )
 					.attr( {
 						role: 'columnheader button',
@@ -374,7 +371,7 @@
 			}
 
 			// add cell to headerList
-			table.config.headerList[headerIndex] = this;
+			config.headerList[headerIndex] = this;
 		} );
 
 		return $tableHeaders;
@@ -391,21 +388,26 @@
 	 */
 	function setHeadersOrder( $headers, sortList, headerToColumns ) {
 		// Loop through all headers to retrieve the indices of the columns the header spans across:
-		$.each( headerToColumns, function( headerIndex, columns ) {
+		$.each( headerToColumns, function ( headerIndex, columns ) {
 
-			$.each( columns, function( i, columnIndex ) {
-				var header = $headers[headerIndex];
+			$.each( columns, function ( i, columnIndex ) {
+				var header = $headers[headerIndex],
+					$header = $( header );
 
 				if ( !isValueInArray( columnIndex, sortList ) ) {
 					// Column shall not be sorted: Reset header count and order.
-					header.order = 0;
-					header.count = 0;
+					$header.data( {
+						order: 0,
+						count: 0
+					} );
 				} else {
 					// Column shall be sorted: Apply designated count and order.
-					$.each( sortList, function( j, sortColumn ) {
+					$.each( sortList, function ( j, sortColumn ) {
 						if ( sortColumn[0] === i ) {
-							header.order = sortColumn[1];
-							header.count = sortColumn[1] + 1;
+							$header.data( {
+								order: sortColumn[1],
+								count: sortColumn[1] + 1
+							} );
 							return false;
 						}
 					} );
@@ -416,8 +418,9 @@
 	}
 
 	function isValueInArray( v, a ) {
-		var l = a.length;
-		for ( var i = 0; i < l; i++ ) {
+		var i,
+			len = a.length;
+		for ( i = 0; i < len; i++ ) {
 			if ( a[i][0] === v ) {
 				return true;
 			}
@@ -427,7 +430,7 @@
 
 	function uniqueElements( array ) {
 		var uniques = [];
-		$.each( array, function( index, elem ) {
+		$.each( array, function ( index, elem ) {
 			if ( elem !== undefined && $.inArray( elem, uniques ) === -1 ) {
 				uniques.push( elem );
 			}
@@ -447,11 +450,11 @@
 	}
 
 	function sortText( a, b ) {
-		return ( (a < b) ? -1 : ((a > b) ? 1 : 0) );
+		return ( ( a < b ) ? -1 : ( ( a > b ) ? 1 : 0 ) );
 	}
 
 	function sortTextDesc( a, b ) {
-		return ( (b < a) ? -1 : ((b > a) ? 1 : 0) );
+		return ( ( b < a ) ? -1 : ( ( b > a ) ? 1 : 0 ) );
 	}
 
 	function multisort( table, sortList, cache ) {
@@ -501,9 +504,9 @@
 
 		// We allow a trailing percent sign, which we just strip. This works fine
 		// if percents and regular numbers aren't being mixed.
-		ts.numberRegex = new RegExp('^(' + '[-+\u2212]?[0-9][0-9,]*(\\.[0-9,]*)?(E[-+\u2212]?[0-9][0-9,]*)?' + // Fortran-style scientific
+		ts.numberRegex = new RegExp( '^(' + '[-+\u2212]?[0-9][0-9,]*(\\.[0-9,]*)?(E[-+\u2212]?[0-9][0-9,]*)?' + // Fortran-style scientific
 		'|' + '[-+\u2212]?' + digitClass + '+[\\s\\xa0]*%?' + // Generic localised
-		')$', 'i');
+		')$', 'i' );
 	}
 
 	function buildDateTable() {
@@ -529,7 +532,7 @@
 
 		// Build RegEx
 		// Any date formated with . , ' - or /
-		ts.dateRegex[0] = new RegExp( /^\s*(\d{1,2})[\,\.\-\/'\s]{1,2}(\d{1,2})[\,\.\-\/'\s]{1,2}(\d{2,4})\s*?/i);
+		ts.dateRegex[0] = new RegExp( /^\s*(\d{1,2})[\,\.\-\/'\s]{1,2}(\d{1,2})[\,\.\-\/'\s]{1,2}(\d{2,4})\s*?/i );
 
 		// Written Month name, dmy
 		ts.dateRegex[1] = new RegExp( '^\\s*(\\d{1,2})[\\,\\.\\-\\/\'\\s]+(' + regex + ')' + '[\\,\\.\\-\\/\'\\s]+(\\d{2,4})\\s*$', 'i' );
@@ -547,7 +550,7 @@
 	 */
 	function explodeRowspans( $table ) {
 		var spanningRealCellIndex, rowSpan, colSpan,
-			cell, i, $tds, $clone, $nextRows,
+			cell, cellData, i, $tds, $clone, $nextRows,
 			rowspanCells = $table.find( '> tbody > tr > [rowspan]' ).get();
 
 		// Short circuit
@@ -563,8 +566,10 @@
 				col = 0,
 				l = this.cells.length;
 			for ( i = 0; i < l; i++ ) {
-				this.cells[i].realCellIndex = col;
-				this.cells[i].realRowIndex = this.rowIndex;
+				$( this.cells[i] ).data( 'tablesorter', {
+					realCellIndex: col,
+					realRowIndex: this.rowIndex
+				} );
 				col += this.cells[i].colSpan;
 			}
 		} );
@@ -574,45 +579,55 @@
 		// Re-sort whenever a rowspanned cell's realCellIndex is changed, because it
 		// might change the sort order.
 		function resortCells() {
+			var cellAData,
+				cellBData,
+				ret;
 			rowspanCells = rowspanCells.sort( function ( a, b ) {
-				var ret = a.realCellIndex - b.realCellIndex;
+				cellAData = $.data( a, 'tablesorter' );
+				cellBData = $.data( b, 'tablesorter' );
+				ret = cellAData.realCellIndex - cellBData.realCellIndex;
 				if ( !ret ) {
-					ret = a.realRowIndex - b.realRowIndex;
+					ret = cellAData.realRowIndex - cellBData.realRowIndex;
 				}
 				return ret;
 			} );
 			$.each( rowspanCells, function () {
-				this.needResort = false;
+				$.data( this, 'tablesorter' ).needResort = false;
 			} );
 		}
 		resortCells();
 
 		function filterfunc() {
-			return this.realCellIndex >= spanningRealCellIndex;
+			return $.data( this, 'tablesorter' ).realCellIndex >= spanningRealCellIndex;
 		}
 
 		function fixTdCellIndex() {
-			this.realCellIndex += colSpan;
+			$.data( this, 'tablesorter' ).realCellIndex += colSpan;
 			if ( this.rowSpan > 1 ) {
-				this.needResort = true;
+				$.data( this, 'tablesorter' ).needResort = true;
 			}
 		}
 
 		while ( rowspanCells.length ) {
-			if ( rowspanCells[0].needResort ) {
+			if ( $.data( rowspanCells[0], 'tablesorter' ).needResort ) {
 				resortCells();
 			}
 
 			cell = rowspanCells.shift();
+			cellData = $.data( cell, 'tablesorter' );
 			rowSpan = cell.rowSpan;
 			colSpan = cell.colSpan;
-			spanningRealCellIndex = cell.realCellIndex;
+			spanningRealCellIndex = cellData.realCellIndex;
 			cell.rowSpan = 1;
 			$nextRows = $( cell ).parent().nextAll();
 			for ( i = 0; i < rowSpan - 1; i++ ) {
 				$tds = $( $nextRows[i].cells ).filter( filterfunc );
 				$clone = $( cell ).clone();
-				$clone[0].realCellIndex = spanningRealCellIndex;
+				$clone.data( 'tablesorter', {
+					realCellIndex: spanningRealCellIndex,
+					realRowIndex: cellData.realRowIndex + i,
+					needResort: true
+				} );
 				if ( $tds.length ) {
 					$tds.each( fixTdCellIndex );
 					$tds.first().before( $clone );
@@ -633,8 +648,8 @@
 			// Build array of key names
 			for ( key in ts.collationTable ) {
 				// Check hasOwn to be safe
-				if ( ts.collationTable.hasOwnProperty(key) ) {
-					keys.push(key);
+				if ( ts.collationTable.hasOwnProperty( key ) ) {
+					keys.push( key );
 				}
 			}
 			if ( keys.length ) {
@@ -649,24 +664,24 @@
 		}
 		ts.rgx = {
 			IPAddress: [
-				new RegExp( /^\d{1,3}[\.]\d{1,3}[\.]\d{1,3}[\.]\d{1,3}$/)
+				new RegExp( /^\d{1,3}[\.]\d{1,3}[\.]\d{1,3}[\.]\d{1,3}$/ )
 			],
 			currency: [
-				new RegExp( /(^[£$€¥]|[£$€¥]$)/),
-				new RegExp( /[£$€¥]/g)
+				new RegExp( /(^[£$€¥]|[£$€¥]$)/ ),
+				new RegExp( /[£$€¥]/g )
 			],
 			url: [
-				new RegExp( /^(https?|ftp|file):\/\/$/),
-				new RegExp( /(https?|ftp|file):\/\//)
+				new RegExp( /^(https?|ftp|file):\/\/$/ ),
+				new RegExp( /(https?|ftp|file):\/\// )
 			],
 			isoDate: [
-				new RegExp( /^\d{4}[\/\-]\d{1,2}[\/\-]\d{1,2}$/)
+				new RegExp( /^\d{4}[\/\-]\d{1,2}[\/\-]\d{1,2}$/ )
 			],
 			usLongDate: [
-				new RegExp( /^[A-Za-z]{3,10}\.? [0-9]{1,2}, ([0-9]{4}|'?[0-9]{2}) (([0-2]?[0-9]:[0-5][0-9])|([0-1]?[0-9]:[0-5][0-9]\s(AM|PM)))$/)
+				new RegExp( /^[A-Za-z]{3,10}\.? [0-9]{1,2}, ([0-9]{4}|'?[0-9]{2}) (([0-2]?[0-9]:[0-5][0-9])|([0-1]?[0-9]:[0-5][0-9]\s(AM|PM)))$/ )
 			],
 			time: [
-				new RegExp( /^(([0-2]?[0-9]:[0-5][0-9])|([0-1]?[0-9]:[0-5][0-9]\s(am|pm)))$/)
+				new RegExp( /^(([0-2]?[0-9]:[0-5][0-9])|([0-1]?[0-9]:[0-5][0-9]\s(am|pm)))$/ )
 			]
 		};
 	}
@@ -681,8 +696,8 @@
 
 	function convertSortList( sortObjects ) {
 		var sortList = [];
-		$.each( sortObjects, function( i, sortObject ) {
-			$.each ( sortObject, function( columnIndex, order ) {
+		$.each( sortObjects, function ( i, sortObject ) {
+			$.each( sortObject, function ( columnIndex, order ) {
 				var orderIndex = ( order === 'desc' ) ? 1 : 0;
 				sortList.push( [parseInt( columnIndex, 10 ), orderIndex] );
 			} );
@@ -693,24 +708,19 @@
 	/* Public scope */
 
 	$.tablesorter = {
-
 			defaultOptions: {
 				cssHeader: 'headerSort',
 				cssAsc: 'headerSortUp',
 				cssDesc: 'headerSortDown',
 				cssChildRow: 'expand-child',
-				sortInitialOrder: 'asc',
 				sortMultiSortKey: 'shiftKey',
-				sortLocaleCompare: false,
 				unsortableClass: 'unsortable',
 				parsers: {},
-				widgets: [],
-				headers: {},
 				cancelSelection: true,
 				sortList: [],
 				headerList: [],
-				selectorHeaders: 'thead tr:eq(0) th',
-				debug: false
+				headerToColumns: [],
+				columnToHeader: []
 			},
 
 			dateRegex: [],
@@ -743,17 +753,13 @@
 					}
 					$table.addClass( 'jquery-tablesorter' );
 
-					// FIXME config should probably not be stored in the plain table node
-					// New config object.
-					table.config = {};
-
-					// Merge and extend.
-					config = $.extend( table.config, $.tablesorter.defaultOptions, settings );
+					// Merge and extend
+					config = $.extend( {}, $.tablesorter.defaultOptions, settings );
 
 					// Save the settings where they read
 					$.data( table, 'tablesorter', { config: config } );
 
-					// Get the CSS class names, could be done else where.
+					// Get the CSS class names, could be done elsewhere
 					sortCSS = [ config.cssDesc, config.cssAsc ];
 					sortMsg = [ mw.msg( 'sort-descending' ), mw.msg( 'sort-ascending' ) ];
 
@@ -778,13 +784,14 @@
 						buildCollationTable();
 
 						// Legacy fix of .sortbottoms
-						// Wrap them inside inside a tfoot (because that's what they actually want to be) &
+						// Wrap them inside a tfoot (because that's what they actually want to be)
 						// and put the <tfoot> at the end of the <table>
-						var $sortbottoms = $table.find( '> tbody > tr.sortbottom' );
+						var $tfoot,
+							$sortbottoms = $table.find( '> tbody > tr.sortbottom' );
 						if ( $sortbottoms.length ) {
-							var $tfoot = $table.children( 'tfoot' );
+							$tfoot = $table.children( 'tfoot' );
 							if ( $tfoot.length ) {
-								$tfoot.eq(0).prepend( $sortbottoms );
+								$tfoot.eq( 0 ).prepend( $sortbottoms );
 							} else {
 								$table.append( $( '<tfoot>' ).append( $sortbottoms ) );
 							}
@@ -792,13 +799,17 @@
 
 						explodeRowspans( $table );
 
-						// try to auto detect column type, and store in tables config
-						table.config.parsers = buildParserCache( table, $headers );
+						// Try to auto detect column type, and store in tables config
+						config.parsers = buildParserCache( table, $headers );
 					}
 
 					// Apply event handling to headers
 					// this is too big, perhaps break it out?
-					$headers.not( '.' + table.config.unsortableClass ).on( 'keypress click', function ( e ) {
+					$headers.not( '.' + config.unsortableClass ).on( 'keypress click', function ( e ) {
+						var cell, $cell, columns, newSortList, i,
+							totalRows,
+							j, s, o;
+
 						if ( e.type === 'click' && e.target.nodeName.toLowerCase() === 'a' ) {
 							// The user clicked on a link inside a table header.
 							// Do nothing and let the default link click action continue.
@@ -822,21 +833,24 @@
 						// cells get event .change() and bubbles up to the <table> here
 						cache = buildCache( table );
 
-						var totalRows = ( $table[0].tBodies[0] && $table[0].tBodies[0].rows.length ) || 0;
+						totalRows = ( $table[0].tBodies[0] && $table[0].tBodies[0].rows.length ) || 0;
 						if ( !table.sortDisabled && totalRows > 0 ) {
-							// Get current column sort order
-							this.order = this.count % 2;
-							this.count++;
+							cell = this;
+							$cell = $( cell );
 
-							var cell, columns, newSortList, i;
+							// Get current column sort order
+							$cell.data( {
+								order: $cell.data( 'count' ) % 2,
+								count: $cell.data( 'count' ) + 1
+							} );
 
 							cell = this;
 							// Get current column index
-							columns = table.headerToColumns[ this.headerIndex ];
+							columns = config.headerToColumns[ $cell.data( 'headerIndex' ) ];
 							newSortList = $.map( columns, function ( c ) {
 								// jQuery "helpfully" flattens the arrays...
-								return [[c, cell.order]];
-							});
+								return [[c, $cell.data( 'order' )]];
+							} );
 							// Index of first column belonging to this header
 							i = columns[0];
 
@@ -851,13 +865,12 @@
 								if ( isValueInArray( i, config.sortList ) ) {
 									// The user has clicked on an already sorted column.
 									// Reverse the sorting direction for all tables.
-									for ( var j = 0; j < config.sortList.length; j++ ) {
-										var s = config.sortList[j],
-											o = config.headerList[s[0]];
+									for ( j = 0; j < config.sortList.length; j++ ) {
+										s = config.sortList[j];
+										o = config.headerList[s[0]];
 										if ( isValueInArray( s[0], newSortList ) ) {
-											o.count = s[1];
-											o.count++;
-											s[1] = o.count % 2;
+											$( o ).data( 'count', s[1] + 1 );
+											s[1] = $( o ).data( 'count' ) % 2;
 										}
 									}
 								} else {
@@ -867,10 +880,10 @@
 							}
 
 							// Reset order/counts of cells not affected by sorting
-							setHeadersOrder( $headers, config.sortList, table.headerToColumns );
+							setHeadersOrder( $headers, config.sortList, config.headerToColumns );
 
 							// Set CSS for headers
-							setHeadersCss( $table[0], $headers, config.sortList, sortCSS, sortMsg, table.columnToHeader );
+							setHeadersCss( $table[0], $headers, config.sortList, sortCSS, sortMsg, config.columnToHeader );
 							appendToTable(
 								$table[0], multisort( $table[0], config.sortList, cache )
 							);
@@ -911,13 +924,13 @@
 
 						// Set each column's sort count to be able to determine the correct sort
 						// order when clicking on a header cell the next time
-						setHeadersOrder( $headers, sortList, table.headerToColumns );
+						setHeadersOrder( $headers, sortList, config.headerToColumns );
 
 						// re-build the cache for the tbody cells
 						cache = buildCache( table );
 
 						// set css for headers
-						setHeadersCss( table, $headers, sortList, sortCSS, sortMsg, table.columnToHeader );
+						setHeadersCss( table, $headers, sortList, sortCSS, sortMsg, config.columnToHeader );
 
 						// sort the table and append it to the dom
 						appendToTable( table, multisort( table, sortList, cache ) );
@@ -934,9 +947,10 @@
 			},
 
 			addParser: function ( parser ) {
-				var l = parsers.length,
+				var i,
+					len = parsers.length,
 					a = true;
-				for ( var i = 0; i < l; i++ ) {
+				for ( i = 0; i < len; i++ ) {
 					if ( parsers[i].id.toLowerCase() === parser.id.toLowerCase() ) {
 						a = false;
 					}
@@ -951,7 +965,7 @@
 				if ( ts.transformTable !== false ) {
 					out = '';
 					for ( p = 0; p < s.length; p++ ) {
-						c = s.charAt(p);
+						c = s.charAt( p );
 						if ( c in ts.transformTable ) {
 							out += ts.transformTable[c];
 						} else {
@@ -965,7 +979,7 @@
 			},
 
 			formatFloat: function ( s ) {
-				var i = parseFloat(s);
+				var i = parseFloat( s );
 				return isNaN( i ) ? 0 : i;
 			},
 
@@ -976,6 +990,15 @@
 
 			clearTableBody: function ( table ) {
 				$( table.tBodies[0] ).empty();
+			},
+
+			getParser: function ( id ) {
+				buildTransformTable();
+				buildDateTable();
+				cacheRegexs();
+				buildCollationTable();
+
+				return getParserById( id );
 			}
 		};
 
@@ -1010,14 +1033,15 @@
 	ts.addParser( {
 		id: 'IPAddress',
 		is: function ( s ) {
-			return ts.rgx.IPAddress[0].test(s);
+			return ts.rgx.IPAddress[0].test( s );
 		},
 		format: function ( s ) {
-			var a = s.split( '.' ),
+			var i, item,
+				a = s.split( '.' ),
 				r = '',
-				l = a.length;
-			for ( var i = 0; i < l; i++ ) {
-				var item = a[i];
+				len = a.length;
+			for ( i = 0; i < len; i++ ) {
+				item = a[i];
 				if ( item.length === 1 ) {
 					r += '00' + item;
 				} else if ( item.length === 2 ) {
@@ -1026,7 +1050,7 @@
 					r += item;
 				}
 			}
-			return $.tablesorter.formatFloat(r);
+			return $.tablesorter.formatFloat( r );
 		},
 		type: 'numeric'
 	} );
@@ -1034,7 +1058,7 @@
 	ts.addParser( {
 		id: 'currency',
 		is: function ( s ) {
-			return ts.rgx.currency[0].test(s);
+			return ts.rgx.currency[0].test( s );
 		},
 		format: function ( s ) {
 			return $.tablesorter.formatDigit( s.replace( ts.rgx.currency[1], '' ) );
@@ -1045,7 +1069,7 @@
 	ts.addParser( {
 		id: 'url',
 		is: function ( s ) {
-			return ts.rgx.url[0].test(s);
+			return ts.rgx.url[0].test( s );
 		},
 		format: function ( s ) {
 			return $.trim( s.replace( ts.rgx.url[1], '' ) );
@@ -1056,11 +1080,11 @@
 	ts.addParser( {
 		id: 'isoDate',
 		is: function ( s ) {
-			return ts.rgx.isoDate[0].test(s);
+			return ts.rgx.isoDate[0].test( s );
 		},
 		format: function ( s ) {
-			return $.tablesorter.formatFloat((s !== '') ? new Date(s.replace(
-			new RegExp( /-/g), '/')).getTime() : '0' );
+			return $.tablesorter.formatFloat( ( s !== '' ) ? new Date( s.replace(
+			new RegExp( /-/g ), '/' ) ).getTime() : '0' );
 		},
 		type: 'numeric'
 	} );
@@ -1068,10 +1092,10 @@
 	ts.addParser( {
 		id: 'usLongDate',
 		is: function ( s ) {
-			return ts.rgx.usLongDate[0].test(s);
+			return ts.rgx.usLongDate[0].test( s );
 		},
 		format: function ( s ) {
-			return $.tablesorter.formatFloat( new Date(s).getTime() );
+			return $.tablesorter.formatFloat( new Date( s ).getTime() );
 		},
 		type: 'numeric'
 	} );
@@ -1079,10 +1103,10 @@
 	ts.addParser( {
 		id: 'date',
 		is: function ( s ) {
-			return ( ts.dateRegex[0].test(s) || ts.dateRegex[1].test(s) || ts.dateRegex[2].test(s ));
+			return ( ts.dateRegex[0].test( s ) || ts.dateRegex[1].test( s ) || ts.dateRegex[2].test( s ) );
 		},
 		format: function ( s ) {
-			var match;
+			var match, y;
 			s = $.trim( s.toLowerCase() );
 
 			if ( ( match = s.match( ts.dateRegex[0] ) ) !== null ) {
@@ -1096,9 +1120,9 @@
 					return '99999999';
 				}
 			} else if ( ( match = s.match( ts.dateRegex[1] ) ) !== null ) {
-				s = [ match[3], '' + ts.monthNames[match[2]], match[1] ];
+				s = [ match[3], String( ts.monthNames[match[2]] ), match[1] ];
 			} else if ( ( match = s.match( ts.dateRegex[2] ) ) !== null ) {
-				s = [ match[3], '' + ts.monthNames[match[1]], match[2] ];
+				s = [ match[3], String( ts.monthNames[match[1]] ), match[2] ];
 			} else {
 				// Should never get here
 				return '99999999';
@@ -1112,8 +1136,7 @@
 				s[2] = '0' + s[2];
 			}
 
-			var y;
-			if ( ( y = parseInt( s[0], 10) ) < 100 ) {
+			if ( ( y = parseInt( s[0], 10 ) ) < 100 ) {
 				// Guestimate years without centuries
 				if ( y < 30 ) {
 					s[0] = 2000 + y;
@@ -1132,7 +1155,7 @@
 	ts.addParser( {
 		id: 'time',
 		is: function ( s ) {
-			return ts.rgx.time[0].test(s);
+			return ts.rgx.time[0].test( s );
 		},
 		format: function ( s ) {
 			return $.tablesorter.formatFloat( new Date( '2000/01/01 ' + s ).getTime() );
@@ -1143,10 +1166,10 @@
 	ts.addParser( {
 		id: 'number',
 		is: function ( s ) {
-			return $.tablesorter.numberRegex.test( $.trim( s ));
+			return $.tablesorter.numberRegex.test( $.trim( s ) );
 		},
 		format: function ( s ) {
-			return $.tablesorter.formatDigit(s);
+			return $.tablesorter.formatDigit( s );
 		},
 		type: 'numeric'
 	} );
